#!/bin/sh

# This is the location of the iptables command
IPTABLES="/sbin/iptables"
TC="/sbin/tc"

INTERNET="eth0"
INTERNET_IP=`/sbin/ifconfig eth0 | grep inet.addr | sed "s/:/ /g" | awk '{print $3}'`
INTERNET_BCAST=`/sbin/ifconfig eth0 |grep inet.addr |sed "s/:/ /g" |awk '{print $5}'`
INTERNET_MASK=`/sbin/ifconfig eth0 |grep inet.addr |sed "s/:/ /g" |awk '{print $7}'`
echo "INTERFACE: $INTERNET IP: $INTERNET_IP BCAST: $INTERNET_BCAST MASK: $INTERNET_MASK"

DMZ="eth1"
DMZ_IP=`/sbin/ifconfig eth1 |grep inet.addr | sed "s/:/ /g" | awk '{print $3}'`
DMZ_BCAST=`/sbin/ifconfig eth1 |grep inet.addr | sed "s/:/ /g" | awk '{print $5}'`
DMZ_MASK=`/sbin/ifconfig eth1 |grep inet.addr |sed "s/:/ /g" |awk '{print $7}'`
echo "INTERFACE: $DMZ IP: $DMZ_IP BCAST: $DMZ_BCAST MASK: $DMZ_MASK"

case "$1" in
   stop)
      echo "Shutting down firewall..."
      $IPTABLES -F
      $IPTABLES -F -t mangle
      $IPTABLES -F -t nat
      $IPTABLES -X
      $IPTABLES -X -t mangle
      $IPTABLES -X -t nat

      $IPTABLES -P INPUT ACCEPT
      $IPTABLES -P OUTPUT ACCEPT
      $IPTABLES -P FORWARD ACCEPT

      $TC qdisc del dev $DMZ root handle 10:0 cbq bandwidth 10Mbit avpkt 1000

      echo "...done"
      ;;
   status)
      echo $"Table: filter"
      iptables --list
      echo $"Table: nat"
      iptables -t nat --list
      echo $"Table: mangle"
      iptables -t mangle --list
      ;;
    restart|reload)
      sh $0 stop
      sh $0 start
      ;;

   start)
    echo "Starting Firewall..."
   
    echo ""


##--------------------------Begin Firewall---------------------------------##


#Clear all IPTABLES-chains
#Flush everything, start from scratch
#$IPTABLES -F
#$IPTABLES -F -t mangle
#$IPTABLES -F -t nat
#$IPTABLES -X
#$IPTABLES -X -t mangle
#$IPTABLES -X -t nat

#Set default policies to DROP
$IPTABLES -P INPUT DROP
$IPTABLES -P OUTPUT DROP
$IPTABLES -P FORWARD DROP


#Reduce DoS'ing ability by reducing timeouts
echo 30 > /proc/sys/net/ipv4/tcp_fin_timeout
echo 2400 > /proc/sys/net/ipv4/tcp_keepalive_time
echo 0 > /proc/sys/net/ipv4/tcp_window_scaling
echo 0 > /proc/sys/net/ipv4/tcp_sack

# Allow the kernel to forward packets and prevent ipspoof
echo 1 > /proc/sys/net/ipv4/ip_forward
echo 1 > /proc/sys/net/ipv4/ip_dynaddr
for f in /proc/sys/net/ipv4/conf/*/rp_filter; do echo 1 > $f; done

########### QOS GOES HERE #################


# allow icmp & syn traffic mark it with value 1
$IPTABLES -A FORWARD -p icmp -o $DMZ -t mangle -j MARK --set-mark 1
$IPTABLES -A FORWARD -p tcp --syn -o $DMZ -t mangle -j MARK --set-mark 1

#mark incoming mail traffic from smtp and pop3 with mark value 2
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport smtp -d 0/0 -t mangle -j MARK --set-mark 2
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport pop3 -d 0/0 -t mangle -j MARK --set-mark 2

#mark incoming  telnet, ftp and ssh traffic with mark value 3
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport 21 -d 0/0 -t mangle -j MARK --set-mark 3
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport 22 -d 0/0 -t mangle -j MARK --set-mark 3
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport 23 -d 0/0 -t mangle -j MARK --set-mark 3

#mark incoming www and Real Server traffic with mark value 4
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport 80 -d 0/0 -t mangle -j MARK --set-mark 4
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport 443 -d 0/0  -t mangle -j MARK --set-mark 4
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport 7070 -d 0/0 -t mangle -j MARK --set-mark 4
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport 554 -d 0/0  -t mangle -j MARK --set-mark 4
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport 8080 -d 0/0  -t mangle -j MARK --set-mark 4
$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport 2687 -d 0/0  -t mangle -j MARK --set-mark 4
$IPTABLES -A FORWARD -p udp -o $DMZ -s 0/0 --dport 554 -d 0/0  -t mangle -j MARK --set-mark 4
$IPTABLES -A FORWARD -p udp -o $DMZ -s 0/0 --dport 8080 -d 0/0  -t mangle -j MARK --set-mark 4
$IPTABLES -A FORWARD -p udp -o $DMZ -s 0/0 --dport 2687 -d 0/0  -t mangle -j MARK --set-mark 4
$IPTABLES -A FORWARD -p udp --destination-port 6970:6999 -o $DMZ -s 0/0 -d 0/0 -t mangle -j MARK --set-mark 4

$IPTABLES -A FORWARD -p udp --dport 6970:6999 -o $DMZ -s 0/0 -d 0/0 -t mangle -j MARK --set-mark 4

#mark incoming mail traffic from smtp and pop3 with mark value 2
#$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport smtp -d 0/0 -t mangle -j MARK --set-mark 2
#$IPTABLES -A FORWARD -p tcp -o $DMZ -s 0/0 --dport pop3 -d 0/0 -t mangle -j MARK --set-mark 2

# allow icmp & syn traffic mark it with value 1
#$IPTABLES -A FORWARD -p icmp -o $DMZ -t mangle -j MARK --set-mark 1
#$IPTABLES -A FORWARD -p tcp --syn -o $DMZ -t mangle -j MARK --set-mark 1
#$IPTABLES -A INPUT -p udp -s 0/0 --dport  53 -j ACCEPT 
#$IPTABLES -A OUTPUT -p udp -d 0/0 --dport 53 -j ACCEPT

# Set up IP FORWARDing and Masquerading

$IPTABLES --table nat --append POSTROUTING --out-interface $INTERNET -j MASQUERADE
$IPTABLES --append FORWARD --in-interface $DMZ -j ACCEPT




# DNAT
# translate incoming ssh and www traffic to my internal hosts
iptables -t nat -A PREROUTING -p tcp --dport 21 -i $INTERNET -j DNAT --to 192.168.0.2:21
iptables -t nat -A PREROUTING -p tcp --dport 22 -i $INTERNET -j DNAT --to 192.168.0.2:22
iptables -t nat -A PREROUTING -p udp --dport 22 -i $INTERNET -j DNAT --to 192.168.0.2:22
iptables -t nat -A PREROUTING -p tcp --dport 23 -i $INTERNET -j DNAT --to 192.168.0.2:23


iptables -t nat -A PREROUTING -p udp --dport 6970:6999 -i $INTERNET -j DNAT --to 192.168.0.2
iptables -t nat -A PREROUTING -p tcp --dport 80 -i $INTERNET -j DNAT --to 192.168.0.2:80
iptables -t nat -A PREROUTING -p tcp --dport 443 -i $INTERNET -j DNAT --to 192.168.0.2:443
iptables -t nat -A PREROUTING -p tcp --dport 8080 -i $INTERNET -j DNAT --to 192.168.0.2:8080
iptables -t nat -A PREROUTING -p tcp --dport 7070 -i $INTERNET -j DNAT --to 192.168.0.2:7070
iptables -t nat -A PREROUTING -p tcp --dport 554 -i $INTERNET -j DNAT --to 192.168.0.2:554
iptables -t nat -A PREROUTING -p tcp --dport 2687 -i $INTERNET -j DNAT --to 192.168.0.2:2687
iptables -t nat -A PREROUTING -p udp --dport 8080 -i $INTERNET -j DNAT --to 192.168.0.2:8080
iptables -t nat -A PREROUTING -p udp --dport 7070 -i $INTERNET -j DNAT --to 192.168.0.2:7070
iptables -t nat -A PREROUTING -p udp --dport 554 -i $INTERNET -j DNAT --to 192.168.0.2:554
iptables -t nat -A PREROUTING -p icmp -i $INTERNET -j DNAT --to 192.168.0.2

#iptables -t nat -A PREROUTING -p tcp -i $INTERNET -j DNAT --to 192.168.0.2
#iptables -t nat -A PREROUTING -p udp -i $INTERNET -j DNAT --to 192.168.0.2
#iptables -t nat -A PREROUTING -i $INTERNET -j DNAT --to 192.168.0.2


#$IPTABLES -t nat -A POSTROUTING -p tcp --sport 554 -o $DMZ -j SNAT --to $INTERNET_IP

$IPTABLES -A INPUT -j ACCEPT
$IPTABLES -A  FORWARD -j ACCEPT
$IPTABLES -A OUTPUT -j ACCEPT

#------End Ruleset------#

add_class() {
# $1=parent class $2=classid $3=hiband $4=lowband $5=handle $6=style
$TC class add dev $DMZ parent $1 classid $2 cbq bandwidth 10Mbit rate $3 allot 1514 weight $4 prio 5  maxburst 20 avpkt 1000 $6
$TC qdisc add dev $DMZ parent $2 sfq quantum 1514b perturb 15
$TC filter add dev $DMZ protocol ip prio 3 handle $5 fw classid $2
}


$TC qdisc add dev $DMZ root handle 10: cbq bandwidth 10Mbit avpkt 1000
$TC class add dev $DMZ parent 10:0 classid 10:1 cbq bandwidth 10Mbit rate 64kbit allot 1514 weight 6.4kbit prio 8 maxburst 20 avpkt 1000 bounded

# first type of traffic ICMP, TCP-SYN, DNS will be marked '1' by the firewall code
# we will give it a bounded bandwidth of 5% of our total incoming bandwidth (10240*0.05=5120.0)
add_class 10:1 10:100 512kbit 51.2kbit 1 bounded
 
# second type of traffic SMTP,POP3 will be marked '2' by the firewalling code
# we will give it a bandwidth of 15% of our total incoming bandwidth (10240*0.15=1536.0)
add_class 10:1 10:200 1536kbit 153.6kbit 2 

# third type of traffic ssh, telnet, ftp will be marked '3' by the firewalling code
# we will give it a bandwidth of 10% of our total incoming bandwidth (10240*0.1=1024.0)
add_class 10:1 10:300 1024kbit 102.4kbit 3 

# last type of traffic is interactive traffic. It will be marked '4' by the firewalling code
# we will give it a bandwidth of 70% of our total incoming bandwidth (10240*0.70=7168.0)
add_class 10:1 10:400 7168kbit 716.8kbit 4 



echo "...done"
echo ""

echo "--> IPTABLES firewall loaded/activated <--"


##--------------------------------End Firewall---------------------------------##
   ;;
   *)
      echo "Usage: firewall (start|stop|restart|status)"
      exit 1
esac

exit 0

