/*
 * copyrights of this program is held by snort. 
 * 
 * Deamon program that automatically send the information to the firewall deamon 
 * so that the firewall scripts can be autonomously.
 *                                     -------------------------
 *                                     written by Angela Cearns
 *                                        acearns[at]yahoo.com
 *  
 * Inputs & advice provided by Dr. Edward Chow <chow@cs.uccs.edu>
 * Inputs & support provided by Ganesh Godavari <gkgodava[at]uccs.edu>
 */

/*
 * I looked into the spo_alert_unixsock.c file the current version of snort doesn't
 * attach packet information into alert. i writing the code for future use -- i guress
 * 
 *  //  we don't log any headers besides eth yet 
 *     if (p->iph) alertpkt.nethdr=(char *)p->iph-(char *)p->pkt;
 *     
 *       switch(p->iph->ip_proto)
 *       {
 *           case IPPROTO_TCP:
 *               if (p->tcph) alertpkt.transhdr=(char *)p->tcph-(char *)p->pkt;
 *               break;
 *
 *           case IPPROTO_UDP:
 *               if (p->udph) alertpkt.transhdr=(char *)p->udph-(char *)p->pkt;
 *               break;
 *
 *           case IPPROTO_ICMP:
 *               if (p->icmph) alertpkt.transhdr=(char *)p->icmph-(char *)p->pkt;
 *               break;
 *
 *           default:
 *               // alertpkt.transhdr is null due to initial bzero 
 *               alertpkt.val|=NO_TRANSHDR;
 *       }
 *       if (p->data) alertpkt.data=p->data - p->pkt;
 *   }
 * 
 */


/* 
 *   Configuration Section
 */

 #define HOSTNAME "titan.uccs.edu"
 #define PORTNO 6779
 #define LOGFILE "/var/log/snort/alert.log"

/* 
 *   End of Configuration Section
 */

#include "snort.h"
#include "msock.h"
#include "misc.h"

int debug = 0;  /* debug flag */

FILE *logfp = (FILE *) NULL;

int analyzePacket ( Alertpkt *alert, FILE *log, char *hostname, int portno)
{
  Packet *p;
  char ipaddr[50];
  int clientfd;

  if (!(alert->val & NOPACKET_STRUCT))
    {
      if ((p = calloc (1, sizeof (Packet))) == NULL)
	{
	  fprintf(stderr," unable to allocate memory for the packet structure");
         return;
	}
      
      p->pkt = alert->pkt;
      p->pkth = &alert->pkth;
      if (alert->dlthdr)
	p->eh = (EtherHdr *) (alert->pkt + alert->dlthdr);
      if (alert->nethdr)
        {
          p->iph = (IPHdr *) (alert->pkt + alert->nethdr);
          if (alert->transhdr)
	    {
             switch (p->iph->ip_proto)
	       {
	       case IPPROTO_TCP:
		 p->tcph = (TCPHdr *) (alert->pkt + alert->transhdr);
		 break;
	       case IPPROTO_UDP:
		 p->udph = (UDPHdr *) (alert->pkt + alert->transhdr);
		 break;
	       case IPPROTO_ICMP:
		 p->icmph = (ICMPHdr *) (alert->pkt + alert->transhdr);
		 break;
	       default:
		 fprintf (stdout, "need to analyse the protcol %d !\n",p->iph->ip_proto);
                    }
	    }
	}
      
      if (alert->data)
	p->data = alert->pkt + alert->data;
    } 
  
#if 0
  printf ("alert->val :: %d NOPACKET_STRUCT %d !(alert->val & NOPACKET_STRUCT) = %d \n",alert->val, NOPACKET_STRUCT, (!(alert->val & NOPACKET_STRUCT)));
#endif

  if (alert->val || (!(alert->val & NOPACKET_STRUCT)))
  {
	  int try =0;
	  
	/* log the information into the logfile  */
        fprintf (log,"%s [%d]\n", alert->alertmsg, alert->event.event_id);
	
	while (try < 10)
	{
	if((clientfd=connectTo(hostname, portno)) == -1) {
	  (void) fprintf(stderr,"Failed to connect to rate limiter \n");
	   try++;
	   continue;
	}
	if(try >= 10)
          return -1;
	else
	  break;
	}
	
	/* send the request to the Rate Limiter */
	sockWrite(clientfd,alert->alertmsg,strlen(alert->alertmsg));
	
        fprintf (stdout,"Data sent ::%s\n", alert->alertmsg);
  }

  
  if (!(alert->val & NOPACKET_STRUCT))
  {
	  
    if (p->iph && (p->tcph || p->udph || p->icmph))
      {
	switch (p->iph->ip_proto)
	  {
	  case IPPROTO_TCP:
	      if (debug)
	       {
	         fprintf(stdout, "TCP from: %s:%d ",inet_ntoa(p->iph->ip_src), ntohs(p->tcph->th_sport));
  	         fprintf(stdout,"to: %s:%d\n", inet_ntoa(p->iph->ip_dst), ntohs(p->tcph->th_dport));
	       }
	    fprintf(log, "TCP from: %s:%d ",inet_ntoa(p->iph->ip_src), ntohs(p->tcph->th_sport));
	    fprintf(log,"to: %s:%d\n", inet_ntoa(p->iph->ip_dst), ntohs(p->tcph->th_dport));
	    break;
	  case IPPROTO_UDP:
	       if (debug)
	       {
                fprintf(stdout,"UDP from: %s:%d ", inet_ntoa(p->iph->ip_src), ntohs (p->udph->uh_sport));
                fprintf(stdout,"to: %s:%d\n", inet_ntoa (p->iph->ip_dst), ntohs (p->udph->uh_dport));
	       }
                fprintf(log,"UDP from: %s:%d ", inet_ntoa(p->iph->ip_src), ntohs (p->udph->uh_sport));
                fprintf(log,"to: %s:%d\n", inet_ntoa (p->iph->ip_dst), ntohs (p->udph->uh_dport));
                break;
	  case IPPROTO_ICMP:
	       if (debug)
	        {
 	         fprintf (stdout,"ICMP type: %d code: %d from: %s ", p->icmph->type, p->icmph->code, inet_ntoa (p->iph->ip_src));
	         fprintf (stdout,"to: %s\n", inet_ntoa (p->iph->ip_dst));
	        }
	    fprintf (log,"ICMP type: %d code: %d from: %s ", p->icmph->type, p->icmph->code, inet_ntoa (p->iph->ip_src));
	    fprintf (log,"to: %s\n", inet_ntoa (p->iph->ip_dst));
	    break;
	  }
      }
    }
  
  close(clientfd);
  fflush(log);
  return 1;
}

void help_msg(char *prog)
{
  fprintf(stdout,"Usage : %s [-d ] -h <hostname> -p <port no> -l <log file name> \n", prog);
  fprintf(stdout, " option d	turn on the debugging mode \n");
  fprintf(stdout, " deafult port no is %d \n",PORTNO);
  return;
}

int main (int argc, char *argv[])
{
 struct sockaddr_un bogus;
 Alertpkt alert;
 char *host;
 char *logfile;
 int  portno=0;
 socklen_t len = sizeof (struct sockaddr_un);
 int sockfd;
 int c;
 struct sockaddr_un snortaddr;

 host = NULL;
 logfile = NULL;
 
 if (
    (logfile = (char *) malloc (sizeof (LOGFILE) + 1)) == NULL ||
    (host  = (char *) malloc(sizeof(HOSTNAME) + 1)) == NULL ) 
 {
   (void) fprintf(stderr," unable to allocate memory for converting relative paths to absolute paths");
    return (-1);
 }
 
(void) sprintf(host,"%s",HOSTNAME);
(void) sprintf(logfile,"%s",LOGFILE);

portno = PORTNO;

 while ((c=getopt(argc,argv,"dhpl")) != EOF)
   {
     switch(c)
       {
       case 'd':
	 debug = 1;
	 if (debug) 
	   (void) fprintf(stdout,"debugging mode turned on \n");
	 break;
           case 'h':
	     host = argv[optind];
	     if (debug)
	       (void) fprintf(stdout,"hostname : %s \n",host);
	     break;
       case 'p':
	 portno = atoi(argv[optind]);
	 if (debug)
	     (void) fprintf(stdout,"port # %d \n",portno);
	 break;
       case 'l':
	 logfile = argv[optind];
	 if (debug) 
	     (void) fprintf(stdout," log file is %s \n",logfile);
	 break;
       default:
	 (void) fprintf(stdout,"unrecognized arg >%s< %d \n", optarg, optind);
	 help_msg(argv[0]);
	 return (-1);
       }
  }
 
   /* open the log file */
 if ( (logfp = fopen(logfile,"a+")) == NULL)
   {
	(void) fprintf(stderr, "Unable to open the log file for logging");
	return -1;
   }

#if 0
 /*
  *  most cool Unix will take care of Zombies if we ignore SIGCHLD
  */
     signal(SIGCHLD,SIG_IGN);
     signal(SIGPIPE,SIG_IGN);

     noCoredump();
     daemonize();
 /* open the socket for checking the connections */
#endif
  unlink (UNSOCK_FILE); /* just a precautionary measure -- not needed */
  
  if ((sockfd = socket (AF_UNIX, SOCK_DGRAM, 0)) < 0)
   {
     (void) fprintf(stderr, "socket creation error \n");
     return (-1);
   }

   bzero (&snortaddr, sizeof (snortaddr));
   snortaddr.sun_family = AF_UNIX;
   strcpy (snortaddr.sun_path, UNSOCK_FILE);

  if (bind (sockfd, (struct sockaddr *) &snortaddr, sizeof (snortaddr)) < 0)
   {
     (void) fprintf(stderr, " unable to bind socket \n");
     return (-1);
   }
	    

 /* check if any alert have been received */
 while (recvfrom (sockfd, (void *) &alert, sizeof (alert), 0, (struct sockaddr *) &bogus, &len) > 0)
    { 
      if (analyzePacket ( &alert, logfp, host, portno) < 0)
	break; 
    }
 
  (void) fprintf(stderr, "recvfrom error \n");
  fclose(logfp);
  close(sockfd);
  unlink (UNSOCK_FILE);
  free(host);
  free(logfile);
  
  return 1;
}


