/*
** Copyright (C) 1998-2002 Martin Roesch <roesch@sourcefire.com>
** Snort Flood Preprocessor Plugin Version 0.01 08/20/2002 -->
** Copyright (C) 2002,2003,2004 Angela Cearns <acearns@yahoo.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#define MODNAME "spp_flood"

#include "spp_flood.h"
#include "rules.h"
#include "log.h"

#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
    #include <sys/time.h>
#else
    #include <time.h>
#endif


#if 0
#define DEBUG 1
#endif

#define MAX(a, b, c) (a > b && a > c)? a : (b > a && b > c)? b : c

/* Definitions for flood types */
struct spp_timeval
{
    time_t tv_sec;
    time_t tv_usec;
};

typedef enum _floodType
{
    sNONE = 0, sUDP = 1, sSYN = 2,sSYNACK = 4, sICMP = 8
} FloodType;

/* Definitions for log levels */
typedef enum _logLevel
{
    lNONE = 0, lFILE = 1, lEXTENDED = 2, lPACKET = 4
} LogLevel;


typedef enum _rateModule
{
    oN = 1, oFF = 0
} RateModule;

/* Structures for keeping track of connection information. */
typedef struct _connectionInfo
{
    FloodType floodType;
    int numberOfTCPConnections;
    int numberOfUDPConnections;
    int numberOfICMPConnections;
    u_short sport;
    u_short dport;
    struct spp_timeval timestamp;
    char tcpFlags[9]; /* Eight flags and a NULL */
    u_char *packetData;
    struct _connectionInfo *prevNode;
    struct _connectionInfo *nextNode;
}               ConnectionInfo;

typedef struct _destinationInfo
{
    struct in_addr daddr;
    int numberOfConnections;
    ConnectionInfo *connectionsList;
    struct _destinationInfo *prevNode;
    struct _destinationInfo *nextNode;
}                DestinationInfo;

typedef struct _sourceInfo
{
    struct in_addr saddr;
    int numberOfConnections;
    int numberOfDestinations;
    int totalNumberOfTCPConnections;
    int totalNumberOfUDPConnections;
    int totalNumberOfICMPConnections;
    int totalNumberOfDestinations;

    struct spp_timeval firstPacketTime;
    struct spp_timeval lastPacketTime;

    int floodDetected;

    /* last time we reported on this source's activities */
    struct spp_timeval reportTime;  
    DestinationInfo *destinationsList;
    u_int32_t event_id;
    struct _sourceInfo *prevNode;
    struct _sourceInfo *nextNode;
}           SourceInfo;

typedef struct _floodList
{
    SourceInfo *listHead;
    SourceInfo *lastSource;
    long numberOfSources;   /* must be as large as address space */
}         FloodList;

typedef struct _rateLimiterDestinationInfo
{
    struct in_addr daddr;
    int numberOfTCPConnections;
    int numberOfUDPConnections;
    int numberOfICMPConnections;
    u_short sport;
    u_short dport;
    struct _rateLimiterDestinationInfo *prevNode;
    struct _rateLimiterDestinationInfo *nextNode;
}                RateLimiterDestinationInfo;


/* Structures for keeping track of rate limiter information. */
typedef struct _rateLimiterSourceInfo
{
    FloodType floodType;
    int numberOfConnections;
    struct in_addr saddr;
    int totalNumberOfTCPConnections;
    int totalNumberOfUDPConnections;
    int totalNumberOfICMPConnections;
    int numberOfDestinations;
    RateLimiterDestinationInfo *destinationsList;
    int floodDetected;
    int subnet;
    int packetThreshold;
    struct spp_timeval firstPacketTime;
    struct spp_timeval lastPacketTime;

    struct spp_timeval reportTime;  /* last time we reported on this
                                     * source's activities */
    u_int32_t event_id;
    struct _rateLimiterSourceInfo *prevNode;
    struct _rateLimiterSourceInfo *nextNode;
}           RateLimiterSourceInfo;

typedef struct _rateList
{
    RateLimiterSourceInfo *listHead;
    RateLimiterSourceInfo *lastSource;
    long numberOfSources;   /* must be as large as address space */
}         RateList;

/* for keeping track of our network's servers */
typedef struct _serverNode  
{
    IpAddrSet *address;
    /*
     * u_long address; u_long netmask;
     */
    char ignoreFlags;
    struct _serverNode *nextNode;
}           ServerNode;



typedef struct _CPConfig
{
    u_int32_t classification;
    u_int32_t priority;
} CPConfig;


/** FUNCTION PROTOTYPES **/
/* Add connection information */
int NewFlood(FloodList *, Packet *, FloodType);

/* Remove connection information */
void ExpireFloodConnections(FloodList *, struct spp_timeval, struct spp_timeval);
void RemoveFloodConnection(ConnectionInfo *);
void RemoveFloodDestination(DestinationInfo *);
void RemoveFloodSource(SourceInfo *);

/* Logging functions */
void LogInfoToSeparateFile(SourceInfo *);

/* Miscellaneous functions */
FloodList *CreateFloodList(void);
int IsFloodServer(Packet *);
FloodType CheckTCPFlagsForFlood(u_char);

/* For Flood-ignorehosts */
void CreateFloodServerList(u_char *);
void FloodParseIP(char *addr);
void FloodIgnoreParseIP(char *addr, ServerNode* server);
IpAddrSet *FloodAllocAddrNode();
IpAddrSet *FloodIgnoreAllocAddrNode(ServerNode * server);

void AlertIntermediateFloodInfo(SourceInfo * currentSource);


/* For Flood-ratelimiting */
RateList *CreateRateList(void);
void ExpireRateConnections(RateList * rateList, struct spp_timeval watchPeriod, struct spp_timeval currentTime);
int updateRateLimiterConnection(RateList *rateList, Packet *p, FloodType floodType);
void AddRateLimiterConnection(RateList *rateList, Packet *p, FloodType floodType, int mask);
void UpdateRateLimiterSource(RateLimiterSourceInfo *source, Packet *p);
void RemoveRateLimiterDestination(RateLimiterDestinationInfo * delDestination);
int checkInRateConnections (RateList *rateList, Packet *p, FloodType floodType);
int rateCheckSubnetIndFlood(RateList * rateList, Packet * p, FloodType floodType);
int rateCheckSubnetFlood(RateList * rateList, Packet * p, FloodType floodType);
void ClearRateConnectionInfoFromFloodSource(RateLimiterSourceInfo *currentSource);


/* Global variables */
FloodList *floodList;
RateList *rateList;
ServerNode *serverList;
FloodType floodToWatch;
CPConfig configdata;

/*u_long homeNet, homeNetMask;*/
IpAddrSet *homeAddr;
int *rateLimitList;
int rateLevels;
char homeFlags;
struct spp_timeval maxTime;
struct spp_timeval reportTime;
long maxConnections;
LogLevel logLevel;
enum _timeFormat
{
    tLOCAL, tGMT
}           timeFormat;
FILE *logFile;
int packetLogSize;      /* Number of data bytes to log per flood
                 * packet */

RateModule rateMod; /* determine if the rate limter module is turned on or off */
/* external globals from rules.c */
extern char *file_name;
extern int file_line;
extern u_int32_t event_id;


ConnectionInfo *NewFloodConnection(Packet * p, FloodType floodType)
{
    ConnectionInfo *newConnection = (ConnectionInfo *) malloc(sizeof(ConnectionInfo));
    if (newConnection == NULL)
      {
          printf("unable to allocate memory for the connection structure \n");
       }

    newConnection->prevNode = NULL;
    newConnection->nextNode = NULL;

    newConnection->floodType = floodType;
    /*
     * timestamp provided by libpcap.  This is available realtime and during
     * -r playback
     */
    newConnection->timestamp.tv_sec = p->pkth->ts.tv_sec;
    newConnection->timestamp.tv_usec = p->pkth->ts.tv_usec;

   /* The ports are already supposed to be in host order from decode.c */
    if (floodType != sICMP)
      {
       newConnection->sport = p->sp;
       newConnection->dport = p->dp;
      }
       else
      {
       newConnection->sport = 0;
       newConnection->dport = 0;
      }



    switch(p->iph->ip_proto)
    {
        case IPPROTO_TCP:
            newConnection->numberOfTCPConnections = 1;
            newConnection->numberOfUDPConnections = 0;
            newConnection->numberOfICMPConnections = 0;
            CreateTCPFlagString(p, newConnection->tcpFlags);

            /* ZDNOTE PACKET LOGGING */
            if(logLevel & lPACKET)
            {
                /*
                 * Determine buffer size = header size + lower(datasize,
                 * packetLogSize)
                 */
                /* Allocate memory */
                /* Copy data */
            }
            break;

        case IPPROTO_UDP:
            strncpy(newConnection->tcpFlags, "\0", 1);
            newConnection->numberOfTCPConnections = 0;
            newConnection->numberOfUDPConnections = 1;
            newConnection->numberOfICMPConnections = 0;

            /* ZDNOTE PACKET LOGGING */
            if(logLevel & lPACKET)
            {
                /*
                 * Determine buffer size = header size + lower(datasize,
                 * packetLogSize)
                 */
                /* Allocate memory */
                /* Copy data */
            }
            break;

        case IPPROTO_ICMP:
            strncpy(newConnection->tcpFlags, "\0", 1);
            newConnection->numberOfTCPConnections = 0;
            newConnection->numberOfUDPConnections = 0;
            newConnection->numberOfICMPConnections = 1;

            /* ZDNOTE PACKET LOGGING */
            if(logLevel & lPACKET)
            {
                /*
                 * Determine buffer size = header size + lower(datasize,
                 * packetLogSize)
                 */
                /* Allocate memory */
                /* Copy data */
            }
            break;

        default:
            /* This should never happen because it's already filtered. */
            FatalError(MODNAME ": NewConnection(): Invalid protocol! (%d)\n", p->iph->ip_proto);
            break;
    }

    return(newConnection);
}

ConnectionInfo *AddFloodConnection(ConnectionInfo * currentConnection, Packet * p, FloodType floodType)
{
    if(currentConnection->nextNode)
        FatalError(MODNAME ":  AddConnection():  Not at end of connection list!");

    currentConnection->nextNode = NewFloodConnection(p, floodType);
    currentConnection->nextNode->prevNode = currentConnection;

    return(currentConnection->nextNode);
}


DestinationInfo *NewFloodDestination(Packet * p, FloodType floodType)
{
    DestinationInfo *newDestination = (DestinationInfo *) malloc(sizeof(DestinationInfo));
    if (newDestination == NULL)
      {
          printf("unable to allocate memory for the destination structure \n");
       }


    newDestination->prevNode = NULL;
    newDestination->nextNode = NULL;
    newDestination->daddr = p->iph->ip_dst;
    newDestination->connectionsList = NewFloodConnection(p, floodType);
    newDestination->numberOfConnections = 1;

    return(newDestination);
}


DestinationInfo *AddFloodDestination(DestinationInfo * currentDestination, Packet * p,
                                FloodType floodType)
{
    if(currentDestination->nextNode)
        FatalError(MODNAME ":  AddDestination():  Not at end of destination list!");

    currentDestination->nextNode = NewFloodDestination(p, floodType);
    currentDestination->nextNode->prevNode = currentDestination;
    return(currentDestination->nextNode);
}


SourceInfo *NewFloodSource(Packet * p, FloodType floodType)
{
    SourceInfo *newSource = (SourceInfo *) malloc(sizeof(SourceInfo));
    if (newSource == NULL)
      {
          printf("unable to allocate memory for the source structure \n");
       }


    newSource->prevNode = NULL;
    newSource->nextNode = NULL;
    newSource->saddr = p->iph->ip_src;
    newSource->numberOfConnections = 1;
    newSource->floodDetected = 0;

#ifdef DEBUG
       printf("New flood source invoked  %s\n", inet_ntoa(newSource->saddr));
#endif


    newSource->firstPacketTime.tv_sec = p->pkth->ts.tv_sec;
    newSource->firstPacketTime.tv_usec = p->pkth->ts.tv_usec;
    newSource->lastPacketTime.tv_sec = p->pkth->ts.tv_sec;
    newSource->lastPacketTime.tv_usec = p->pkth->ts.tv_usec;

    if(floodType == sUDP)
    {
        newSource->totalNumberOfTCPConnections = 0;
        newSource->totalNumberOfUDPConnections = 1;
        newSource->totalNumberOfICMPConnections = 0;
#ifdef DEBUG
        printf(MODNAME ": NewSource(): %s->totalnumberOfUDPConnections = 1, TCP = 0\n", inet_ntoa(newSource->saddr));
#endif

    }
    else if(floodType == sICMP)
    {
        newSource->totalNumberOfTCPConnections = 0;
        newSource->totalNumberOfUDPConnections = 0;
        newSource->totalNumberOfICMPConnections = 1;

#ifdef DEBUG
        printf(MODNAME ": NewSource(): %s->totalnumberOfICMPConnections = 1, TCP = 0\n", inet_ntoa(newSource->saddr));
#endif

    }
    else
    {
        newSource->totalNumberOfTCPConnections = 1;
        newSource->totalNumberOfUDPConnections = 0;
        newSource->totalNumberOfICMPConnections = 0;

#ifdef DEBUG
        printf(MODNAME ": NewSource(): %s->totalNumberOfTCPConnections = 1, UDP = 0\n", inet_ntoa(newSource->saddr));
#endif

    }

    newSource->destinationsList = NewFloodDestination(p, floodType);
    newSource->numberOfDestinations = 1;
    newSource->totalNumberOfDestinations = 1;

    return(newSource);
}


SourceInfo *AddFloodSource(SourceInfo * currentSource, Packet * p, FloodType floodType)
{
    if(currentSource->nextNode)
        FatalError(MODNAME ":  AddSource():  Not at end of source list!");
#ifdef DEBUG
    printf(" AddFloodSource Invoked");
#endif

    currentSource->nextNode = NewFloodSource(p, floodType);
    currentSource->nextNode->prevNode = currentSource;

    return(currentSource->nextNode);
}


void RemoveFloodConnection(ConnectionInfo * delConnection)
{
    /*
     * If there is a prev and/or next node, make them point to the proper
     * places.  Otherwise, just delete this node.
     */
    if(delConnection->prevNode || delConnection->nextNode)
    {
        if(delConnection->prevNode)
        {
            delConnection->prevNode->nextNode = delConnection->nextNode;
        }
        else if(delConnection->nextNode)
        {
            delConnection->nextNode->prevNode = NULL;
        }
        if(delConnection->nextNode)
        {
            delConnection->nextNode->prevNode = delConnection->prevNode;
        }
        else if(delConnection->prevNode)
        {
            delConnection->prevNode->nextNode = NULL;
        }
    }
    free(delConnection);
}


void RemoveFloodDestination(DestinationInfo * delDestination)
{
    /*
     * If there is a prev and/or next node, make them point to the proper
     * places.  Otherwise, just delete this node.
     */
    if(delDestination->prevNode || delDestination->nextNode)
    {
        if(delDestination->prevNode)
        {
            delDestination->prevNode->nextNode = delDestination->nextNode;
        }
        else if(delDestination->nextNode)
        {
            delDestination->nextNode->prevNode = NULL;
        }
        if(delDestination->nextNode)
        {
            delDestination->nextNode->prevNode = delDestination->prevNode;
        }
        else if(delDestination->prevNode)
        {
            delDestination->prevNode->nextNode = NULL;
        }
    }
    free(delDestination);
}


void RemoveFloodSource(SourceInfo * delSource)
{
    /*
     * If there is a prev and/or next node, make them point to the proper
     * places.  Otherwise, just delete this node.
     */
    if(delSource->prevNode || delSource->nextNode)
    {
        if(delSource->prevNode)
        {
            delSource->prevNode->nextNode = delSource->nextNode;
        }
        else if(delSource->nextNode)
        {
            delSource->nextNode->prevNode = NULL;
        }
        if(delSource->nextNode)
        {
            delSource->nextNode->prevNode = delSource->prevNode;
        }
        else if(delSource->prevNode)
        {
            delSource->prevNode->nextNode = NULL;
        }
    }
    free(delSource);
}

/* Go through each connection and remove any connections that are old.  If
   the removal of a node makes a parent node empty, remove that node, all
   the way back to the root.
*/
void ExpireFloodConnections(FloodList * floodList, struct spp_timeval watchPeriod,
                       struct spp_timeval currentTime)
{
    SourceInfo *currentSource = floodList->listHead, *tmpSource;
    DestinationInfo *currentDestination, *tmpDestination;
    ConnectionInfo *currentConnection, *tmpConnection;


#ifdef DEBUG
   printf("Expire Flood Connections invoked floodList->listHead):: %d  noofsources :: %d \n",floodList->listHead, floodList->numberOfSources);
#endif

    /* Empty list. Get out of here. */
    if(!floodList->listHead)
        return;


    while(currentSource != NULL)
    {
        /*
         * If this source host is flooding us, we don't want to lose any
         * connections so go back to top.
         */
        if(currentSource->floodDetected)
        {
            currentSource = currentSource->nextNode;
            continue;
        }

        currentDestination = currentSource->destinationsList;

        while(currentDestination)
        {
           if(currentDestination->connectionsList != NULL)
              currentConnection = currentDestination->connectionsList;
           else
              currentConnection = NULL;

            while(currentConnection)
            {
                if(currentConnection->timestamp.tv_sec + watchPeriod.tv_sec < currentTime.tv_sec)
                {
                    /* Expire the connection */
                    tmpConnection = currentConnection;
                    currentConnection = currentConnection->nextNode;

                    /*
                     * If this is the first connection, we need to update
                     * connectionsList.
                     */
                    if(tmpConnection->prevNode == NULL)
                    {
                        currentDestination->connectionsList = tmpConnection->nextNode;
                    }
                    if(tmpConnection->floodType == sUDP)
                    {
                        currentSource->totalNumberOfUDPConnections -= tmpConnection->numberOfUDPConnections;

#ifdef DEBUG
                        printf(MODNAME ": ExpireConnections(): %s->totalNumberOfUDPConnections-- (%d)\n",
                               inet_ntoa(currentSource->saddr), currentSource->totalNumberOfUDPConnections);
#endif

                    }
                     else if(tmpConnection->floodType == sICMP)
                    {
                        currentSource->totalNumberOfICMPConnections -= tmpConnection->numberOfICMPConnections;

#ifdef DEBUG
                        printf(MODNAME ": ExpireConnections(): %s->totalNumberOfICMPConnections-- (%d)\n",
                               inet_ntoa(currentSource->saddr), currentSource->totalNumberOfICMPConnections);
#endif

                    }
                    else
                    {
                        currentSource->totalNumberOfTCPConnections -= tmpConnection->numberOfTCPConnections;

#ifdef DEBUG
                        printf(MODNAME ": ExpireConnections(): %s->totalNumberOfTCPConnections-- (%d)\n",
                               inet_ntoa(currentSource->saddr), currentSource->totalNumberOfTCPConnections);
#endif

                    }

                    RemoveFloodConnection(tmpConnection);
                    currentSource->numberOfConnections--  ;
                    currentDestination->numberOfConnections--;

                }
                else
                {
                    currentConnection = currentConnection->nextNode;
                }
            }

            tmpDestination = currentDestination;
            currentDestination = currentDestination->nextNode;

            if(tmpDestination->numberOfConnections == 0)
            {
		if(tmpDestination->prevNode == NULL)
                {
                    currentSource->destinationsList = tmpDestination->nextNode;
                }
                currentSource->numberOfDestinations--;
                RemoveFloodDestination(tmpDestination);
            }
        }

        tmpSource = currentSource;
        currentSource = currentSource->nextNode;

        if(tmpSource->numberOfDestinations == 0)
        {
            /* If this is the first source, we need to update floodList. */
            if(tmpSource->prevNode == NULL)
            {
                /* This is fine, even if tmpSource->nextNode is NULL */
                floodList->listHead = tmpSource->nextNode;
            }else if(tmpSource->prevNode == NULL && tmpSource->nextNode == NULL)
              {

                 floodList->listHead = NULL;

              }

            RemoveFloodSource(tmpSource);
            floodList->numberOfSources--;
        }
    }

    if(floodList->numberOfSources == 0)
    {
        floodList->listHead = NULL;
    }
}


void ExtractFloodHeaderInfo(Packet * p, struct in_addr * saddr, 
                       struct in_addr * daddr, u_short * sport, 
                       u_short * dport, u_short mode)
{
    /*
     * This function seems kinda silly now that I don't have to do protocol
     * checks to use the proper protocol headers to get the port, but I think
     * it still makes it easier and I don't have to worry about something
     * changing later.
     */
if (mode)
    {
    *sport = p->sp;
    *dport = p->dp;
    }
else
   {
    *sport = 0;
    *dport = 0;
   }

    *saddr = p->iph->ip_src;
    *daddr = p->iph->ip_dst;
}



/*
    Add the connection information and return the
    new number of connections for this host
*/
int NewFlood(FloodList * floodList, Packet * p, FloodType floodType)
{
    SourceInfo *currentSource = floodList->listHead;
    DestinationInfo *currentDestination;
    ConnectionInfo *currentConnection;
    int matchFound = 0;

    struct in_addr saddr;
    struct in_addr daddr;
    u_short sport;
    u_short dport;

    /* If list is empty, create the list and add this entry. */
    if(!floodList->listHead)
    {
        floodList->listHead = NewFloodSource(p, floodType);
        floodList->numberOfSources = 1;
        floodList->lastSource = floodList->listHead;
        return(floodList->listHead->numberOfConnections);
    }

    if (floodType != sICMP )
        ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 1);
    else
        ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 0);

#ifdef DEBUG
printf("protocol type %d, saddr %x , daddr %x, sport %d dport %d\n", floodType, ntohl(saddr.s_addr), ntohl(daddr.s_addr), ntohs(sport), ntohs(dport));
#endif

    while(!matchFound)
    {
        if(currentSource->saddr.s_addr == saddr.s_addr)
        {
            currentDestination = currentSource->destinationsList;

            if(currentSource->destinationsList == NULL)
            {
                currentSource->destinationsList = NewFloodDestination(p, floodType);
                currentSource->numberOfConnections++;

                if(floodType == sUDP)
                {
                    currentSource->totalNumberOfUDPConnections++;

#ifdef DEBUG
                    printf(MODNAME ": NewFlood(): %s->totalNumberOfUDPConnections++ (%d)\n",
                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfUDPConnections);
#endif

                }
                else if(floodType == sICMP)
                {
                    currentSource->totalNumberOfICMPConnections++;

#ifdef DEBUG
                    printf(MODNAME ": NewFlood(): %s->totalNumberOfICMPConnections++ (%d)\n",
                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfICMPConnections);
#endif

                }
                else
                {
                    currentSource->totalNumberOfTCPConnections++;

#ifdef DEBUG
                    printf(MODNAME ": NewFlood(): %s->totalNumberOfTCPConnections++ (%d)\n",
                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfTCPConnections);
#endif

                }

                currentSource->numberOfDestinations++;
                matchFound = 1;
            }

            currentDestination = currentSource->destinationsList;

            while(!matchFound)
            {
                if(currentDestination->daddr.s_addr == daddr.s_addr)
                {
                    currentConnection = currentDestination->connectionsList;

                    while(!matchFound)
                    {

                        /*
                         * There should be error checking for
                         * currentConnection == NULL, but that should never
                         * happen.
                         */

                        if(currentConnection == NULL)
                            FatalError(MODNAME ": currentConnection is NULL!!!??\n");

                        if((currentConnection->dport == dport) && (currentConnection->floodType == floodType))
                        {
                          currentConnection->sport = sport;

                        if(floodType == sUDP)
                        {
                            currentSource->totalNumberOfUDPConnections++;
			    currentConnection->numberOfUDPConnections++;

#ifdef DEBUG
			   printf("number of udp connections %d \n", currentConnection->numberOfUDPConnections);
                            printf(MODNAME ": Match NewFlood(): %s->totalNumberOfUDPConnections++ (%d)\n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfUDPConnections);
#endif

                        }
                        else if(floodType == sICMP)
                        {
                            currentSource->totalNumberOfICMPConnections++;
			    currentConnection->numberOfICMPConnections++;

#ifdef DEBUG
                            printf(MODNAME ": Match NewFlood(): %s->totalNumberOfICMPConnections++ (%d) dport = %d \n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfICMPConnections, dport);
#endif

                        }
                        else
                        {
                            currentSource->totalNumberOfTCPConnections++;
			    currentConnection->numberOfTCPConnections++;

#ifdef DEBUG
                            printf(MODNAME ": Match NewFlood(): %s->totalNumberOfTCPConnections++ (%d)\n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfTCPConnections);
#endif

                        }

                            matchFound = 1;
                        }
                        else
                        {
                            /*
                             * If not at end of list, keep going, otherwise
                             * create a node.
                             */
                            if(!currentConnection->nextNode)
                            {
                                currentConnection = AddFloodConnection(currentConnection, p, floodType);
                                currentSource->numberOfConnections++;

                                if(floodType == sUDP)
                                {
                                    currentSource->totalNumberOfUDPConnections++;

#ifdef DEBUG
                                    printf(MODNAME ": NewFlood(): %s->numberOfUDPConnections++ (%d)\n",
                                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfUDPConnections);
#endif
                        if(floodType == sUDP)
                        {
                            currentSource->totalNumberOfUDPConnections++;

#ifdef DEBUG
                            printf(MODNAME ": NewFlood(): %s->totalNumberOfUDPConnections++ (%d)\n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfUDPConnections);
#endif

                        }
                                }
                                else if(floodType == sICMP)
                                {
                                    currentSource->totalNumberOfICMPConnections++;

#ifdef DEBUG
                                    printf(MODNAME ": NewFlood(): %s->totalNumberOfICMPConnections++ (%d)\n",
                                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfICMPConnections);
#endif
                        if(floodType == sICMP)
                        {
                            currentSource->totalNumberOfICMPConnections++;

#ifdef DEBUG
                            printf(MODNAME ": NewFlood(): %s->totalNumberOfICMPConnections++ (%d)\n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfICMPConnections);
#endif

                        }
                                }
                                else
                                {
                                    currentSource->totalNumberOfTCPConnections++;

#ifdef DEBUG
                                    printf(MODNAME ": NewFlood(): %s->totalNumberOfTCPConnections++ (%d)\n",
                                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfTCPConnections);
#endif

                                }

                                currentDestination->numberOfConnections++;
                                matchFound = 1;
                            }
                            else
                                currentConnection = currentConnection->nextNode;
                        }
                    }
                }
                else
                {
                    if(!currentDestination->nextNode)
                    {
                        currentDestination = AddFloodDestination(currentDestination, p, floodType);
                        currentSource->numberOfConnections++;

                        if(floodType == sUDP)
                        {
                            currentSource->totalNumberOfUDPConnections++;

#ifdef DEBUG
                            printf(MODNAME ": NewFlood(): %s->totalNumberOfUDPConnections++ (%d)\n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfUDPConnections);
#endif

                        }
                        else if(floodType == sICMP)
                        {
                            currentSource->totalNumberOfICMPConnections++;

#ifdef DEBUG
                            printf(MODNAME ": NewFlood(): %s->totalNumberOfICMPConnections++ (%d)\n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfICMPConnections);
#endif

                        }
                        else
                        {
                            currentSource->totalNumberOfTCPConnections++;

#ifdef DEBUG
                            printf(MODNAME ": NewFlood(): %s->totalNumberOfTCPConnections++ (%d)\n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfTCPConnections);
#endif

                        }

                        currentSource->numberOfDestinations++;
                        currentSource->totalNumberOfDestinations++;
                        matchFound = 1;
                    }
                    else
                        currentDestination = currentDestination->nextNode;
                }
            }
        }
        else
        {
            if(!currentSource->nextNode)
            {
                currentSource = AddFloodSource(currentSource, p, floodType);
                currentSource->numberOfConnections = 1;

                if(floodType == sUDP)
                {
                    currentSource->totalNumberOfUDPConnections = 1;

#ifdef DEBUG
                    printf(MODNAME ": NewFlood(): %s->totalNumberOfUDPConnections = 1 \n", inet_ntoa(currentSource->saddr));
#endif

                }
                else if(floodType == sICMP)
                {
                    currentSource->totalNumberOfICMPConnections = 1;

#ifdef DEBUG
                    printf(MODNAME ": NewFlood(): %s->totalNumberOfICMPConnections = 1 \n", inet_ntoa(currentSource->saddr));
#endif

                }
                else
                {
                    currentSource->totalNumberOfTCPConnections = 1;

#ifdef DEBUG
                    printf(MODNAME ": NewFlood(): %s->totalNumberOfTCPConnections = 1\n", inet_ntoa(currentSource->saddr));
#endif

                }

                floodList->numberOfSources++;
                matchFound = 1;
            }
            else
                currentSource = currentSource->nextNode;
        }
    }

    floodList->lastSource = currentSource;

    return MAX(currentSource->totalNumberOfTCPConnections, currentSource->totalNumberOfUDPConnections, currentSource->totalNumberOfICMPConnections);

}

FloodList *CreateFloodList(void)
{
    FloodList *newList = (FloodList *) malloc(sizeof(FloodList));
    if (newList == NULL)
      {
          printf("unable to allocate memory for the newList structure \n");
       }


#ifdef DEBUG
       printf("flood list initialized\n");
#endif

    newList->listHead = NULL;
    newList->lastSource = NULL;
    newList->numberOfSources = 0;

    return(newList);
}


void ClearConnectionInfoFromFloodSource(SourceInfo * currentSource)
{
    DestinationInfo *currentDestination, *tmpDestination;
    ConnectionInfo *currentConnection, *tmpConnection;

    currentDestination = currentSource->destinationsList;
    while(currentDestination)
    {
        currentConnection = currentDestination->connectionsList;
        while(currentConnection)
        {
            tmpConnection = currentConnection;
            currentConnection = currentConnection->nextNode;

            if(tmpConnection->floodType == sUDP)
            {
                currentSource->totalNumberOfUDPConnections -= tmpConnection->numberOfUDPConnections;

#ifdef DEBUG
                printf(MODNAME ": ClearConnectionInfoFromFloodSource(): %s->totalNumberOfUDPConnections-- (%d)\n",
                       inet_ntoa(currentSource->saddr), currentSource->totalNumberOfUDPConnections);
#endif

            }
            else if(tmpConnection->floodType == sICMP)
            {
                currentSource->totalNumberOfICMPConnections -= tmpConnection->numberOfICMPConnections;

#ifdef DEBUG
                printf(MODNAME ": ClearConnectionInfoFromFloodSource(): %s->numberOfICMPConnections-- (%d)\n",
                       inet_ntoa(currentSource->saddr), currentSource->totalNumberOfICMPConnections);
#endif

            }
            else
            {
                currentSource->totalNumberOfTCPConnections -= tmpConnection->numberOfTCPConnections;

#ifdef DEBUG
                printf(MODNAME ": ClearConnectionInfoFromFloodSource(): %s->totalNumberOfTCPConnections-- (%d)\n",
                       inet_ntoa(currentSource->saddr), currentSource->totalNumberOfTCPConnections);

#endif

            }

            RemoveFloodConnection(tmpConnection);
            currentDestination->numberOfConnections--;
            currentSource->numberOfConnections--;
        }

        tmpDestination = currentDestination;
        currentDestination = currentDestination->nextNode;
        printf(" Expected removing flood Destination %u and the link is from %u to %u \n", tmpDestination, tmpDestination->prevNode, tmpDestination->nextNode);
        RemoveFloodDestination(tmpDestination);
        printf("removing flood Destination %u and the link is from %u to %u \n", tmpDestination, tmpDestination->prevNode, tmpDestination->nextNode);
        currentSource->numberOfDestinations--;
    }
    currentSource->destinationsList = NULL;
}

/* Check if packet originated from a machine we have been told to ignore
   SYN and UDP "floods" from, presumably because it's a server.
*/
int IsFloodServer(Packet * p)
{
    ServerNode *currentServer = serverList;

#ifdef DEBUG 
    char sourceIP[16], ruleIP[16], ruleNetMask[16];
#endif

    while(currentServer)
    {
        /*
         * Return 1 if the source addr is in the serverlist, 0 if nothing is
         * found.
         */
        if(CheckAddrPort(currentServer->address, 0, 0, p,
                         (ANY_SRC_PORT | currentServer->ignoreFlags), CHECK_SRC))
        {

#ifdef DEBUG
            memset(sourceIP, '\0', 16);
            memset(ruleIP, '\0', 16);
            memset(ruleNetMask, '\0', 16);
            strncpy(sourceIP, inet_ntoa(p->iph->ip_src), 15);
            strncpy(ruleIP, inet_ntoa(*(struct in_addr *) & (currentServer->address->ip_addr)), 14);
            strncpy(ruleNetMask, inet_ntoa(*(struct in_addr *) & (currentServer->address->netmask)), 15);

            printf(MODNAME ": IsFloodServer():  Server %s found in %s/%s!\n", sourceIP, ruleIP, ruleNetMask);
#endif

            return(1);
        }
        currentServer = currentServer->nextNode;
    }
    return(0);
}



int inSubnet(u_long saddr, u_long subnet, int bitShift)
{
#ifdef DEBUG
  printf(" saddr = %X subnet = %X \n",(u_long) saddr,(u_long) subnet);
#endif
    bitShift = 32 - bitShift;

    /* check for match in the /bitShift subnet  */
    if ( (saddr >> bitShift) == (subnet >> bitShift))
        return 1;

   return 0;
}


int CheckSubnetIndFlood(FloodList * floodList, Packet * p, FloodType floodType, int *subnet)
{
    struct in_addr saddr;
    struct in_addr daddr;
    u_short sport;
    u_short dport;
    SourceInfo *currentSource = floodList->listHead;
    DestinationInfo *currentDestination;
    ConnectionInfo *currentConnection;
    int subnet8;
    int subnet10;
    int subnet16;
    int retVal;


   if (floodType != sICMP )
        ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 1);
    else
        ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 0);

    subnet8 = subnet10 = subnet16 = 0;

    while(currentSource)
    {
        if( inSubnet((u_long) ntohl(currentSource->saddr.s_addr), (u_long) ntohl(saddr.s_addr), 8) == 1)
          {
            currentDestination = currentSource->destinationsList;

            while (currentDestination)
            {
               if(currentDestination->daddr.s_addr == daddr.s_addr)
                {
		  currentConnection = currentDestination->connectionsList;
                  while(currentConnection)
                  {
		    if((currentConnection->dport == dport) && (currentConnection->floodType == floodType))
                     {
                     if (floodType == sUDP)
         	             subnet8 += currentConnection->numberOfUDPConnections;
                     else if(floodType == sICMP)
         	             subnet8 += currentConnection->numberOfICMPConnections;
                     else
         	            subnet8 += currentConnection->numberOfTCPConnections;
	/*	printf(" subnet 8 tcp syn value = %d totalnumber of connections %d \n",subnet8,currentSource->totalNumberOfTCPConnections);*/
                     }
		   currentConnection = currentConnection->nextNode;
                  }
                }
	   currentDestination = currentDestination->nextNode;
          }
         } 
#if 0 
	else if( inSubnet((u_long) ntohl(currentSource->saddr.s_addr), (u_long) ntohl(saddr.s_addr), 10) == 1)
          {
            currentDestination = currentSource->destinationsList;
            
            while (currentDestination)
            {
               if(currentDestination->daddr.s_addr == daddr.s_addr)
                {
		  currentConnection = currentDestination->connectionsList;
                  while(currentConnection)
                  {
		    if((currentConnection->dport == dport) && (currentConnection->floodType == floodType))
                     {
                     if (floodType == sUDP)
         	             subnet10 += currentConnection->numberOfUDPConnections;
                     else if(floodType == sICMP)
         	             subnet10 += currentConnection->numberOfICMPConnections;
                     else
         	            subnet10 += currentConnection->numberOfTCPConnections;
		printf(" subnet 10 tcp syn value = %d totalnumber of connections %d \n",subnet8,currentSource->totalNumberOfTCPConnections);
                     }
		   currentConnection = currentConnection->nextNode;
                  }
                }
	       currentDestination = currentDestination->nextNode;
            }
           }
#endif
#if 0
else if( inSubnet((u_long) ntohl(currentSource->saddr.s_addr), (u_long) ntohl(saddr.s_addr), 16) == 1)
          {
            currentDestination = currentSource->destinationsList;
            
            while (currentDestination)
            {
               if(currentDestination->daddr.s_addr == daddr.s_addr)
                {
		  currentConnection = currentDestination->connectionsList;
                  while(currentConnection)
                  {
		    if((currentConnection->dport == dport) && (currentConnection->floodType == floodType))
                     {
                     if (floodType == sUDP)
         	             subnet16 += currentConnection->numberOfUDPConnections;
                     else if(floodType == sICMP)
         	             subnet16 += currentConnection->numberOfICMPConnections;
                     else
         	            subnet16 += currentConnection->numberOfTCPConnections;
		printf(" subnet 16 tcp syn value = %d totalnumber of connections %d \n",subnet8,currentSource->totalNumberOfTCPConnections);
                     }
		   currentConnection = currentConnection->nextNode;
                  }
                }
	   currentDestination = currentDestination->nextNode;
          }
         }
#endif
	  currentSource = currentSource->nextNode;
    }

  retVal = subnet8;
	*subnet = 32 -8;
#if 0  
  retVal = (subnet8>subnet10)?subnet8:subnet10;

   if (retVal == subnet8)
       *subnet = 32 - 8;
   else if (retVal == subnet10)
       *subnet = 32 - 10;
#endif
#if 0
   else if (retVal == subnet16)
       *subnet = 32 - 16;
#endif

   return retVal;

}

/*
 * i have no idea what all i can do with this i can do some anomaly detection work here -- i guess 
 * first let me focus on getting the number of connections where the 8 10 and 16 subnet match.
 */

int CheckSubnetFlood(FloodList * floodList, Packet * p, FloodType floodType, int *subnet)
{
    struct in_addr saddr;
    struct in_addr daddr;
    u_short sport;
    u_short dport;
    SourceInfo *currentSource = floodList->listHead;
    int subnet8;
    int subnet10;
    int subnet16;
    int retVal;


   if (floodType != sICMP )
        ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 1);
    else
        ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 0);

    subnet8 = subnet10 = subnet16 = 0;

    while(currentSource)
    {
        
        if( inSubnet((u_long) ntohl(currentSource->saddr.s_addr), (u_long) ntohl(saddr.s_addr), 8) == 1)
          {
                     if (floodType == sUDP)
         	             subnet8 += currentSource->totalNumberOfUDPConnections;
                     else if(floodType == sICMP)
         	             subnet8 += currentSource->totalNumberOfICMPConnections;
                     else
         	            subnet8 += currentSource->totalNumberOfTCPConnections;

         }
#if 0	
	else if( inSubnet((u_long) ntohl(currentSource->saddr.s_addr), (u_long) ntohl(saddr.s_addr), 10) == 1)
	{
                     if (floodType == sUDP)
         	             subnet10 += currentSource->totalNumberOfUDPConnections;
                     else if(floodType == sICMP)
         	             subnet10 += currentSource->totalNumberOfICMPConnections;
                     else
         	            subnet10 += currentSource->totalNumberOfTCPConnections;
	}
#endif
#if 0
        else if( inSubnet((u_long) ntohl(currentSource->saddr.s_addr), (u_long) ntohl(saddr.s_addr), 16) == 1)
        {
                     if (floodType == sUDP)
         	             subnet16 += currentSource->totalNumberOfUDPConnections;
                     else if(floodType == sICMP)
         	             subnet16 += currentSource->totalNumberOfICMPConnections;
                     else
         	            subnet16 += currentSource->totalNumberOfTCPConnections;
         }
#endif
        

	  currentSource = currentSource->nextNode;
    }

    retVal = subnet8;
    *subnet = 32 - 8;
    
#if 0
   retVal = MAX(subnet8, subnet10, subnet16);
   retVal = (subnet8>subnet10)?subnet8:subnet10;

   if (retVal == subnet8)
       *subnet = 32 - 8;
   else if (retVal == subnet10)
       *subnet = 32 - 10;

   else if (retVal == subnet16)
       *subnet = 32 - 16;
#endif
    return retVal;


}

void FloodPreprocFunction(Packet * p)
{
    SourceInfo *currentSource;
    FloodType floodType;
    struct spp_timeval currTime;
    char logMessage[180];
    int numConnections;
    int postConnections;
    Event event;
    RateLimiterSourceInfo *currentRateSource;


    floodType = sNONE;

    /* Only do processing on IP Packets */
    if(p->iph == NULL)
    {
        return;
    }

#ifdef DEBUG
    printf ("packet source = %x \n", ntohl(p->iph->ip_src.s_addr));
#endif

    /*
     * Here we check if it is a protocol we are watching and if it is a
     * destination we are watching.  If either fails, we return abruptly.
     */
    switch(p->iph->ip_proto)
    {
        case IPPROTO_TCP:
            if(p->tcph == NULL)
            {
                return;
            }
#ifdef DEBUG
            printf("spp_flood: Got TCP pkt\n");
#endif
            floodType = CheckTCPFlagsForFlood(p->tcph->th_flags);
            break;

        case IPPROTO_UDP:
            floodType = sUDP;
           
#ifdef DEBUG
            printf("spp_flood: Got UDP pkt\n");
#endif
            break;

        case IPPROTO_ICMP:
#if 0
        printf("source address is %s ", inet_ntoa(p->iph->ip_src));
        printf(" destination is %s\n", inet_ntoa(p->iph->ip_dst));
#endif
#ifdef DEBUG
            printf("spp_flood: Got ICMP pkt\n");
#endif
            floodType = sICMP;

            break;

        default:
            /* The packet isn't a protocol we watch, so get out of here. */
            return;         /*** RETURN ***/
            break;
    }


    /* if the flood type is unknown */
    if(!floodType)
        return;
    

  /*
     * The checks above are faster, so now that we know this packet is
     * interesting we'll check the address.
     */
    if(!CheckAddrPort(homeAddr, 0, 0, p,
                      (ANY_DST_PORT | homeFlags), CHECK_DST))
    {
        return;
    }


    /*
     * if its from the ignore hosts please ignore this
     */

    if(IsFloodServer(p))
    {
        floodType &= ~(sSYN | sUDP | sICMP);
    }



#ifdef DEBUG
    printf ("floodType %d & floodTowatch %d = %d \n",floodType, floodToWatch, (floodType & floodToWatch));

#endif

    if(floodType & floodToWatch)
    {

        currTime.tv_sec = p->pkth->ts.tv_sec;
        currTime.tv_usec = p->pkth->ts.tv_usec;
        ExpireFloodConnections(floodList, maxTime, currTime);
        numConnections = NewFlood(floodList, p, floodType);

        /* Timestamp info for statistics */
        floodList->lastSource->lastPacketTime = currTime;

       /* individual connections match found */
        if((numConnections > maxConnections) || (floodType & ~(sSYN | sUDP | sICMP)))
        {
            if(!floodList->lastSource->floodDetected)
            {
	         if (rateMod == oN)
                  {
                   if (checkInRateConnections (rateList, p, floodType))
                    {
                     goto SNITCH;
                    }
                    else
                      AddRateLimiterConnection(rateList, p, floodType, 32);
                  }

                    if(pv.alert_interface_flag)
                    {
                        sprintf(logMessage, MODNAME 
                                ": FLOOD DETECTED on %s from %s/32"
                                " (THRESHOLD %ld connections exceeded in %ld seconds)",
                                PRINT_INTERFACE(pv.interfaces[0]), 
                                inet_ntoa(floodList->lastSource->saddr), maxConnections,
                                (long int) (currTime.tv_sec - 
                                            floodList->lastSource->firstPacketTime.tv_sec));
                    }
                    else
                    {
                        sprintf(logMessage, 
                                MODNAME ": FLOOD DETECTED from %s/32"
                                " (THRESHOLD %ld connections exceeded in %ld seconds)",
                                inet_ntoa(floodList->lastSource->saddr), maxConnections,
                                (long int) (currTime.tv_sec - 
                                            floodList->lastSource->firstPacketTime.tv_sec));
                    }


                CallAlertFuncs(NULL , logMessage, NULL, &event);
                floodList->lastSource->floodDetected = 1;
                floodList->lastSource->reportTime = currTime;
                floodList->lastSource->event_id = event_id;
            }

       }
else  /* check for subnet flooding ;)*/
             {
              int subnet;

            /* check for sybnet to individual host*/
            numConnections = CheckSubnetIndFlood(floodList, p, floodType, &subnet);
            if((numConnections > maxConnections) || (floodType & ~(sSYN | sUDP | sICMP)))
              {
	         if (rateMod == oN)
                  {
                   if (checkInRateConnections (rateList, p, floodType))
                    {
                     goto SNITCH;
                    }
                    else
                      AddRateLimiterConnection(rateList, p, floodType, subnet);
                  }

            if(!floodList->lastSource->floodDetected)
                {
                    if(pv.alert_interface_flag)
                    {
                        sprintf(logMessage, MODNAME 
                                ":indi SubNet FLOOD DETECTED on %s from %s"
                                " (THRESHOLD %d connections exceeded in %ld seconds)",
                                PRINT_INTERFACE(pv.interfaces[0]), 
                                inet_ntoa(floodList->lastSource->saddr), numConnections,
                                (long int) (currTime.tv_sec - 
                                            floodList->lastSource->firstPacketTime.tv_sec));
                    }
                    else
                    {
			sprintf(logMessage,
                                MODNAME "indi: SubNet FLOOD DETECTED from %s/%d to "
                                "%s (THRESHOLD %d connections exceeded in %ld seconds)",
                                inet_ntoa(floodList->lastSource->saddr),subnet,
                                inet_ntoa(p->iph->ip_dst), numConnections,
                                (long int) (currTime.tv_sec -
                                            floodList->lastSource->firstPacketTime.tv_sec));

                    }


                CallAlertFuncs(NULL , logMessage, NULL, &event);
                floodList->lastSource->floodDetected = 1;
                floodList->lastSource->reportTime = currTime;
                floodList->lastSource->event_id = event_id;
               }
            }
            else /* check for subnet to subnet */
            {
            numConnections = CheckSubnetFlood(floodList, p, floodType, &subnet);
            if((numConnections > maxConnections) || (floodType & ~(sSYN | sUDP | sICMP)))
              {
	         if (rateMod == oN)
                  {
                   if (checkInRateConnections (rateList, p, floodType))
                    {
                     goto SNITCH;
                    }
                    else
                      AddRateLimiterConnection(rateList, p, floodType, subnet);
                  }

            if(!floodList->lastSource->floodDetected)
                {
                    if(pv.alert_interface_flag)
                    {
                        sprintf(logMessage, MODNAME 
                                ":SubNet FLOOD DETECTED on %s from %s"
                                " (THRESHOLD %d connections exceeded in %ld seconds)",
                                PRINT_INTERFACE(pv.interfaces[0]), 
                                inet_ntoa(floodList->lastSource->saddr), numConnections,
                                (long int) (currTime.tv_sec - 
                                            floodList->lastSource->firstPacketTime.tv_sec));
                    }
                    else
                    {
			sprintf(logMessage,
                                MODNAME "subnet: SubNet FLOOD DETECTED from %s/%d to "
                                "%s (THRESHOLD %d connections exceeded in %ld seconds)",
                                inet_ntoa(floodList->lastSource->saddr),subnet,
                                inet_ntoa(p->iph->ip_dst), numConnections,
                                (long int) (currTime.tv_sec -
                                            floodList->lastSource->firstPacketTime.tv_sec));

                    }


                CallAlertFuncs(NULL , logMessage, NULL, &event);
                floodList->lastSource->floodDetected = 1;
                floodList->lastSource->reportTime = currTime;
                floodList->lastSource->event_id = event_id;
               }
            }
           }
         }

SNITCH:	 
	 if (rateMod == oN)
	 {
           /* here u go rate limiting */
	    postConnections = updateRateLimiterConnection(rateList,p,floodType);

        /* Timestamp info for statistics */
        if (rateList->lastSource)
           rateList->lastSource->lastPacketTime = currTime;

#if 0
            printf("postConnections == %d... \n", postConnections);
#endif

            ExpireRateConnections(rateList, reportTime, currTime); 
            if (rateList->lastSource != NULL && rateList->lastSource->packetThreshold)
             {
              if(postConnections > rateList->lastSource->packetThreshold && rateLevels > rateList->lastSource->floodDetected)
               {
                /* do some logging */
                sprintf(logMessage, 
                                MODNAME " %d :rate FLOOD DETECTED from %s/%d"
                                " (THRESHOLD %ld connections exceeded in %ld seconds)", postConnections,
                                inet_ntoa(rateList->lastSource->saddr), rateList->lastSource->subnet, rateList->lastSource->packetThreshold,
                                (long int) (currTime.tv_sec - rateList->lastSource->firstPacketTime.tv_sec));

               CallAlertFuncs(NULL , logMessage, NULL, &event);
               UpdateRateLimiterSource(rateList->lastSource,p);
               rateList->lastSource->reportTime = currTime;
               rateList->lastSource->event_id = event_id;

              }
#if 0
              else if( rateCheckSubnetIndFlood(rateList, p, floodType) > rateList->lastSource->packetThreshold && rateLevels > rateList->lastSource->floodDetected)
                {
                   /* do some logging */
                 sprintf(logMessage, 
                                 MODNAME ":rateind FLOOD DETECTED from %s/%d"
                                 " (THRESHOLD %ld connections exceeded in %ld seconds)",
                                 inet_ntoa(rateList->lastSource->saddr), rateList->lastSource->subnet, rateList->lastSource->packetThreshold,
                                 (long int) (currTime.tv_sec - 
                                             rateList->lastSource->firstPacketTime.tv_sec));

                CallAlertFuncs(NULL , logMessage, NULL, &event);
                UpdateRateLimiterSource(rateList->lastSource,p);
                rateList->lastSource->reportTime = currTime;
                rateList->lastSource->event_id = event_id;
              }
              else if( rateCheckSubnetFlood(rateList, p, floodType) > rateList->lastSource->packetThreshold && rateLevels > rateList->lastSource->floodDetected)
                {
                   /* do some logging */
                 sprintf(logMessage, 
                                 MODNAME ":ratesub FLOOD DETECTED from %s/%d"
                                 " (THRESHOLD %ld connections exceeded in %ld seconds)",
                                 inet_ntoa(rateList->lastSource->saddr), rateList->lastSource->subnet, rateList->lastSource->packetThreshold,
                                 (long int) (currTime.tv_sec - 
                                             rateList->lastSource->firstPacketTime.tv_sec));

                CallAlertFuncs(NULL , logMessage, NULL, &event);
                UpdateRateLimiterSource(rateList->lastSource,p);
                rateList->lastSource->reportTime = currTime;
                rateList->lastSource->event_id = event_id;
              }
#endif

            } /* end of lastSource end detection */
          }	/* end of while */
            /* here u end rate limiting ;-)*/



        /* See if there's anyone we can snitch on.  ;)  */
        currentSource = floodList->listHead;
        while(currentSource)
        {
            if(currentSource->floodDetected)
            {
                if(currentSource->reportTime.tv_sec + maxTime.tv_sec < currTime.tv_sec)
                {
                    if(currentSource->numberOfConnections == 0)
                    {
                        /* Flood stopped.  Clear flag. */
                        sprintf(logMessage, MODNAME ": End of Flood from %s: TOTAL time(%lds) hosts(%d) TCP(%d) UDP(%d) ICMP(%d)",
                                inet_ntoa(currentSource->saddr),
                                (long int) (currentSource->lastPacketTime.tv_sec - currentSource->firstPacketTime.tv_sec),
                                currentSource->totalNumberOfDestinations,
                                currentSource->totalNumberOfTCPConnections,
                                currentSource->totalNumberOfUDPConnections,
                                currentSource->totalNumberOfICMPConnections);

                        CallAlertFuncs(NULL , logMessage, NULL, &event);
                        currentSource->floodDetected = 0;
                    }
                    else
                    {
                        ClearConnectionInfoFromFloodSource(currentSource);
                        currentSource->reportTime = currTime;
                    }
                }
            }
            currentSource = currentSource->nextNode;
        }

 if (rateMod == oN)
  {
        /* See if there's anyone we can snitch on.  ;)  */
        currentRateSource = rateList->listHead;
        while(currentRateSource)
        {
            if(currentRateSource->floodDetected)
            {
                if(currentRateSource->reportTime.tv_sec + maxTime.tv_sec < currTime.tv_sec)
                {

printf("currentRateSource->numberOfConnections == %d \n",currentRateSource->numberOfConnections);


                    if(currentRateSource->numberOfConnections == 0)
                    {
                        /* Flood stopped.  Clear flag. */
                        sprintf(logMessage, MODNAME ": End of Rate Flood from %s/%d: TOTAL time(%lds) hosts(%d) TCP(%d) UDP(%d) ICMP(%d)",
                                inet_ntoa(currentRateSource->saddr), currentRateSource->subnet,
                                (long int) (currentRateSource->lastPacketTime.tv_sec - currentRateSource->firstPacketTime.tv_sec),
                                currentRateSource->numberOfDestinations,
                                currentRateSource->totalNumberOfTCPConnections,
                                currentRateSource->totalNumberOfUDPConnections,
                                currentRateSource->totalNumberOfICMPConnections);

                        CallAlertFuncs(NULL , logMessage, NULL, &event);
                        currentRateSource->floodDetected = 0;
                    }
                    else
                    {
                        ClearRateConnectionInfoFromFloodSource(currentRateSource);

			currentRateSource->destinationsList = NULL;

                        currentRateSource->reportTime = currTime;
                    }
                }
            }
            currentRateSource = currentRateSource->nextNode;
        }

}

    }


}



void SetupFlood(void)
{
    RegisterPreprocessor("flood", FloodInit);
}


void FloodInit(u_char * args)
{
    ParseFloodArgs(args);
    floodList = CreateFloodList();

    /*
     * We set serverList to NULL here so if flood-ignorehosts is used it
     * must be set after flood.  This is necessary to make sure we don't
     * check an empty list.
     */
    serverList = NULL;
    AddFuncToPreprocList(FloodPreprocFunction);
}



FloodType CheckTCPFlagsForFlood(u_char th_flags)
{
    u_char th_flags_cleaned;
    FloodType flood = sNONE;

    /*
     * Strip off the reserved bits for the testing, but flag that a flood is
     * being done.
     */
    th_flags_cleaned = th_flags & ~(R_RES1 | R_RES2);

    /* I'm disabling reserved bits flood detection until we can get a
     * handle on ECN, we're seeing far too many false positives with
     * this code right now -MFR
     */ 
    /*if(th_flags != th_flags_cleaned)
    {
        flood = sRESERVEDBITS;
    }*/
    /*
     * Most TCP packets will have the ACK bit set, so split that out quickly.
     * Any floodss which use SYN  must be added to this part.
     * Otherwise, it goes in the !ACK section. In addition, anything that is
     * !ACK && !SYN eventually gets flagged as something.  This is to
     * hopefully detect new flood types.
     */
    if(th_flags_cleaned & R_ACK)
    {

        /*
         * This is from ipt_unclean.c from the netfilter package.  We are
         * allowing packets which are valid and flagging the rest as
         * INVALIDACK, if it's not already listed as some other flood.
         */
        switch(th_flags_cleaned)
        {
            case (R_ACK):
            case (R_FIN | R_ACK):
            case (R_RST | R_ACK):
            case (R_ACK | R_PSH):
            case (R_ACK | R_URG):
            case (R_ACK | R_URG | R_PSH):
            case (R_FIN | R_ACK | R_PSH):
            case (R_FIN | R_ACK | R_URG):
            case (R_FIN | R_ACK | R_URG | R_PSH):
            case (R_RST | R_ACK | R_PSH):   /* Found through numerous false
                             * alerts. */
                /* Nothing.  This is legitimate traffic. */
                break;
            case (R_SYN | R_ACK):
                flood |= sSYNACK;
                break;

        }
    }
    else
    {
        switch(th_flags_cleaned)
        {
            case R_SYN:
#ifdef DEBUG
                printf("spp_flood: SYN packet\n");
#endif
                flood |= sSYN;
                break;

        }
    }

#ifdef DEBUG
                if (flood == sSYN)
                printf("spp_flood: SYN packet %d\n",flood);
#endif

    return(flood);
}


void ParseFloodArgs(u_char * args)
{
    char **toks;
    int numToks;

    char *logFileName;

#ifdef DEBUG
    printf(MODNAME ": FloodArgs(): %s\n", args);
#endif

    logLevel = lNONE;

    if(!args)
    {
        FatalError(MODNAME ": ERROR: %s (%d) => flood configuration format:  address/mask ports seconds [logfile]\n", file_name, file_line);
    }

    /* the '\\' sets the string escape delimiter - MFR */
#ifdef WIN32
    toks = mSplit(args, " ", 6, &numToks, 0);
#else
    toks = mSplit(args, " ", 6, &numToks, '\\');    /* ZDNOTE What does the
                                                     * '\\' do? */
#endif

    if((numToks < 3) || (numToks > 6))
    {
        FatalError(MODNAME ": ERROR: %s (%d) => flood configuration format:  address/mask ports seconds [logfile]\n", file_name, file_line);
    }

    maxConnections = atoi(toks[1]);
    maxTime.tv_sec = atoi(toks[2]);
    maxTime.tv_usec = 0;

    FloodParseIP(toks[0]);
    /* ParseIP(toks[0], homeAddr); */

    /*
     * Now we use the default log directory if provided from the command line
     * (-l).
     */
    if(numToks == 4)
    {
#ifdef WIN32
        logFileName = (char *) calloc(strlen(toks[3]) + 1 + 1, 1);
        strncpy(logFileName, toks[3], strlen(toks[3]));
#else
        if(pv.log_dir && (*toks[3] != '/'))
        {
            if(*(pv.log_dir + strlen(pv.log_dir) - 1) != '/')
            {
                logFileName = chrootdir == NULL ?
                    (char *) calloc(strlen(pv.log_dir) + strlen(toks[3]) + 1 + 1, 1) :
                    (char *) calloc(strlen(pv.log_dir) + strlen(toks[3]) + strlen(chrootdir) + 1 + 1, 1);

                if(chrootdir)
                    strncpy(logFileName, chrootdir, strlen(chrootdir) + 1);
                strncat(logFileName, pv.log_dir, strlen(pv.log_dir) + 1);
                strncat(logFileName, "/", 1);
                strncat(logFileName, toks[3], strlen(toks[3]));
            }
            else
            {
                logFileName = chrootdir == NULL ?
                    (char *) calloc(strlen(pv.log_dir) + strlen(toks[3]) + 1, 1) :
                    (char *) calloc(strlen(pv.log_dir) + strlen(toks[3]) + strlen(chrootdir) + 1, 1);
                if(chrootdir)
                    strncpy(logFileName, chrootdir, strlen(chrootdir) + 1);
                strncat(logFileName, pv.log_dir, strlen(pv.log_dir) + 1);
                strncat(logFileName, toks[3], strlen(toks[3]));
            }
        }
        else
        {
            logFileName = chrootdir == NULL ?
                (char *) calloc(strlen(toks[3]) + 1, 1) :
                (char *) calloc(strlen(toks[3]) + strlen(chrootdir) + 1, 1);
            if(chrootdir)
                strncpy(logFileName, chrootdir, strlen(chrootdir) + 1);
            strncat(logFileName, toks[3], strlen(toks[3]) + 1);
        }
#endif  /* WIN32 */

#ifdef DEBUG
        printf(MODNAME ": logFileName = %s\n", logFileName);
#endif

        logFile = fopen(logFileName, "a+");
        if(!logFile)
        {
            perror("fopen");
            FatalError(MODNAME ": logfile open error (%s)\n", logFileName);
        }

        logLevel |= lFILE;
    }
    /* How about some error detecting? :) */
    if(maxConnections == 0 || maxTime.tv_sec == 0)
    {
        FatalError(MODNAME ": ERROR: %s (%d) => flood configuration format:  address/mask ports seconds [logfile]\n", file_name, file_line);
    }
    
      floodToWatch = sUDP | sSYN | sICMP;
     
    logLevel |= lEXTENDED;

    packetLogSize = 100;

    if(pv.use_utc == 1)
    {
        timeFormat = tGMT;
        if(!pv.quiet_flag)
            printf("Using GMT time\n");
    }
    else
    {
        timeFormat = tLOCAL;
        if(!pv.quiet_flag)
            printf("Using LOCAL time\n");
    }
}

void SetupFloodIgnoreHosts(void)
{
    RegisterPreprocessor("flood-ignorehosts", FloodIgnoreHostsInit);
}

void FloodIgnoreHostsInit(u_char * args)
{
    CreateFloodServerList(args);
}


/* Well, it seems we are ignoring more than just servers now.  We're also
   ignoring SYN and UDP flood from our own networks.  I guess this is okay.
   Most networks have a soft, chewy center, anyway.  Besides, this
   makes the coding easier! ;)
*/
void CreateFloodServerList(u_char * servers)
{
    char **toks;
    int num_toks;
    int num_servers = 0;
    ServerNode *currentServer;
    int i;

#ifdef DEBUG
    char ruleIP[16], ruleNetMask[16];
#endif

    currentServer = NULL;
    serverList = NULL;

    if(servers == NULL)
    {
        FatalError(MODNAME ": ERROR: %s (%d)=> No arguments to flood-ignorehosts preprocessor!\n", file_name, file_line);
    }
    /* tokenize the argument list */
    toks = mSplit(servers, " ", 31, &num_toks, '\\');

    /* convert the tokens and place them into the server list */
    for(num_servers = 0; num_servers < num_toks; num_servers++)
    {
        if(currentServer != NULL)
        {
            currentServer->nextNode = (ServerNode *) calloc(sizeof(ServerNode), sizeof(char));
            currentServer = currentServer->nextNode;
        }
        else
        {
            currentServer = (ServerNode *) calloc(sizeof(ServerNode), sizeof(char));
            serverList = currentServer;
        }

#ifdef DEBUG
        printf(MODNAME ": CreateFloodServerList(): Adding server %s\n", toks[num_servers]);
#endif
        /* currentServer->ignoreFlags = 0; */

          FloodIgnoreParseIP(toks[num_servers], currentServer);
        /* ParseIP(toks[num_servers], &currentServer->address); */

#ifdef DEBUG
        memset(ruleIP, '\0', 16);
        memset(ruleNetMask, '\0', 16);
        strncpy(ruleIP, inet_ntoa(*(struct in_addr *) & currentServer->address->ip_addr), 15);
        strncpy(ruleNetMask, inet_ntoa(*(struct in_addr *) & currentServer->address->netmask), 15);
        printf(MODNAME ": CreateFloodServerList(): Added server %s/%s\n", ruleIP, ruleNetMask);
#endif

        currentServer->nextNode = NULL;
    }

    for(i = 0; i < num_toks; i++)
    {
        free(toks[i]);
    }
}



void FloodParseIP(char *addr)
{
    char **toks;
    int num_toks;
    int i;
    IpAddrSet *tmp_addr;
    char *tmp;

    if(*addr == '!')
    {
        homeFlags |= EXCEPT_DST_IP;
        addr++;
    }

    if(*addr == '$')
    {
        if((tmp = VarGet(addr + 1)) == NULL)
        {
            FatalError("ERROR %s (%d) => Undefined variable %s\n", file_name, 
                       file_line, addr);
        }
    }
    else
    {
        tmp = addr;
    }

    if (*tmp == '[')
    {
        *(strrchr(tmp, (int)']')) = 0; /* null out the en-bracket */

        toks = mSplit(tmp+1, ",", 128, &num_toks, 0);

        for(i = 0; i < num_toks; i++)
        {
            tmp_addr = FloodAllocAddrNode();

            ParseIP(toks[i], tmp_addr);
        }

        for(i=0;i<num_toks;i++)
            free(toks[i]);
    } 
    else
    {
        tmp_addr = FloodAllocAddrNode();
        ParseIP(tmp, tmp_addr);
    }
}


/***** AlertIntermediateInfo() *****
  Log number of flood packets and types to standard alert mechanism.
*/
void AlertIntermediateFloodInfo(SourceInfo * currentSource)
{
    char logMessage[160];
    Event event;

    sprintf(logMessage, 
            MODNAME ": spp_flood status from %s: %d connections "
            "across %d hosts: TCP(%d), UDP(%d), ICMP(%d)",
            inet_ntoa(currentSource->saddr), 
            currentSource->numberOfConnections,
            currentSource->numberOfDestinations,
            currentSource->totalNumberOfTCPConnections, 
            currentSource->totalNumberOfUDPConnections,
            currentSource->totalNumberOfICMPConnections);

    CallAlertFuncs(NULL, logMessage, NULL, &event);

    return;
}

void FloodIgnoreParseIP(char *addr, ServerNode* server)
{
    char **toks;
    int num_toks;
    int i;
    IpAddrSet *tmp_addr;
    int global_negation_flag;
    char *tmp;

    if(addr == NULL)
    {
        FatalError("ERROR %s(%d) => Undefined address in flood-ignorehosts directive\n", file_name, file_line);
    }

    if(*addr == '!')
    {
        global_negation_flag = 1;
        addr++;
    }

    if(*addr == '$')
    {
        if((tmp = VarGet(addr + 1)) == NULL)
        {
            FatalError("ERROR %s (%d) => Undefined variable %s\n", file_name, 
                       file_line, addr);
        }
    }
    else
    {
        tmp = addr;
    }

    if (*tmp == '[')
    {
        *(strrchr(tmp, (int)']')) = 0; /* null out the en-bracket */

        toks = mSplit(tmp+1, ",", 128, &num_toks, 0);

        for(i = 0; i < num_toks; i++)
        {
            tmp_addr = FloodIgnoreAllocAddrNode(server);

            ParseIP(toks[i], tmp_addr);
        }

        for(i=0;i<num_toks;i++)
            free(toks[i]);
    } 
    else
    {
        tmp_addr = FloodIgnoreAllocAddrNode(server);
        
        ParseIP(tmp, tmp_addr);
    }
}
IpAddrSet *FloodAllocAddrNode()
{
    IpAddrSet *idx;     /* IP struct indexing pointer */

    if(homeAddr == NULL)
    {
        homeAddr = (IpAddrSet *) calloc(sizeof(IpAddrSet), sizeof(char));

        if(homeAddr == NULL)
        {
            FatalError("[!] ERROR: Unable to allocate space for flood IP addr\n");
        }

        return homeAddr;
    }

    idx = homeAddr;

    while(idx->next != NULL)
    {
        idx = idx->next;
    }

    idx->next = (IpAddrSet *) calloc(sizeof(IpAddrSet), sizeof(char));

    idx = idx->next;

    if(idx == NULL)
    {
        FatalError("[!] ERROR: Unable to allocate space for flood IP address\n");
    }

    return idx;
}

IpAddrSet *FloodIgnoreAllocAddrNode(ServerNode * server)
{
    IpAddrSet *idx;     /* IP struct indexing pointer */

    if(server->address == NULL)
    {
        server->address = (IpAddrSet *) calloc(sizeof(IpAddrSet), sizeof(char));

        if(server->address == NULL)
        {
            FatalError("[!] ERROR: Unable to allocate space for flood IP addr\n");
        }
        return server->address;
    }
    idx = server->address;

    while(idx->next != NULL)
    {
        idx = idx->next;
    }

    idx->next = (IpAddrSet *) calloc(sizeof(IpAddrSet), sizeof(char));

    idx = idx->next;

    if(idx == NULL)
    {
        FatalError("[!] ERROR: Unable to allocate space for flood IP address\n");
    }
    return idx;
}

RateList *CreateRateList(void)
{
    RateList *newList = (RateList *) malloc(sizeof(RateList));
    if (newList == NULL)
      {
          printf("unable to allocate memory for the newList Rate structure \n");
       }

#ifdef DEBUG
       printf("Rate list initialized\n");
#endif

    newList->listHead = NULL;
    newList->lastSource = NULL;
    newList->numberOfSources = 0;

    return(newList);
}

void UpdateRateLimiterSource(RateLimiterSourceInfo *source, Packet *p)
{
  RateLimiterDestinationInfo *currentDestination;
  RateLimiterDestinationInfo *tmpDestination;

   source->firstPacketTime.tv_sec = p->pkth->ts.tv_sec;
   source->firstPacketTime.tv_usec = p->pkth->ts.tv_usec;
   source->lastPacketTime.tv_sec = p->pkth->ts.tv_sec;
   source->lastPacketTime.tv_usec = p->pkth->ts.tv_usec;
   source->floodDetected++;
   source->packetThreshold = rateLimitList[source->floodDetected];
   source->numberOfConnections = 0;
   source->totalNumberOfTCPConnections = 0;
   source->totalNumberOfICMPConnections = 0;
   source->totalNumberOfUDPConnections = 0;

   currentDestination = source->destinationsList;
   while(currentDestination)
   {
     tmpDestination = currentDestination->nextNode;
     RemoveRateLimiterDestination(currentDestination);
     currentDestination = tmpDestination;          
   }

  source->destinationsList = NULL;
           
   
}


RateLimiterDestinationInfo *NewRateLimiterDestination(Packet * p, FloodType floodType)
{
    RateLimiterDestinationInfo *newDestination = (RateLimiterDestinationInfo *) malloc(sizeof(RateLimiterDestinationInfo));
    if (newDestination == NULL)
      {
          printf("unable to allocate memory for the newDestination Rate structure \n");
       }

    newDestination->prevNode = NULL;
    newDestination->nextNode = NULL;
    newDestination->daddr = p->iph->ip_dst;

    if (floodType != sICMP)
      {
       newDestination->sport = p->sp;
       newDestination->dport = p->dp;
      }
       else
      {
       newDestination->sport = 0;
       newDestination->dport = 0;
      }

    switch(p->iph->ip_proto)
     {
        case IPPROTO_TCP:
            newDestination->numberOfTCPConnections = 1;
            newDestination->numberOfUDPConnections = 0;
            newDestination->numberOfICMPConnections = 0;
            break;

        case IPPROTO_UDP:
            newDestination->numberOfTCPConnections = 0;
            newDestination->numberOfUDPConnections = 1;
            newDestination->numberOfICMPConnections = 0;
            break;

        case IPPROTO_ICMP:
            newDestination->numberOfTCPConnections = 0;
            newDestination->numberOfUDPConnections = 0;
            newDestination->numberOfICMPConnections = 1;
            break;

        default:
            /* This should never happen because it's already filtered. */
            FatalError(MODNAME ": NewSubnetFloodDestination(): Invalid protocol! (%d)\n", p->iph->ip_proto);
            break;
    }

    return(newDestination);
}


RateLimiterDestinationInfo *AddRateLimiterDestination(RateLimiterDestinationInfo * currentDestination, Packet * p, FloodType floodType)
{
    if(currentDestination->nextNode)
        FatalError(MODNAME ":  AddRateLimiterDestination:  Not at end of connection list!");

    currentDestination->nextNode = NewRateLimiterDestination(p, floodType);
    currentDestination->nextNode->prevNode = currentDestination;

    return(currentDestination->nextNode);
}

void RemoveRateLimiterDestination(RateLimiterDestinationInfo * delDestination)
{
    /*
     * If there is a prev and/or next node, make them point to the proper
     * places.  Otherwise, just delete this node.
     */
    if(delDestination->prevNode || delDestination->nextNode)
    {
        if(delDestination->prevNode)
        {
            delDestination->prevNode->nextNode = delDestination->nextNode;
        }
        else if(delDestination->nextNode)
        {
            delDestination->nextNode->prevNode = NULL;
        }
        if(delDestination->nextNode)
        {
            delDestination->nextNode->prevNode = delDestination->prevNode;
        }
        else if(delDestination->prevNode)
        {
            delDestination->prevNode->nextNode = NULL;
        }
    }

    free(delDestination);
}

void ClearRateConnectionInfoFromFloodSource(RateLimiterSourceInfo *currentSource)
{
    int matchFound = 0;

    RateLimiterDestinationInfo *currentDestination, *tmpDestination;

    currentDestination = currentSource->destinationsList;


    if (currentSource->destinationsList == NULL)
       return;


    while(!matchFound)
    {
      tmpDestination = currentDestination;

      if(currentDestination == NULL)
      {
        matchFound = 1;
        currentSource->destinationsList = NULL;
      }else if(currentDestination->nextNode == NULL) {
        matchFound = 1;
             
             currentSource->numberOfConnections--;
             currentSource->numberOfDestinations--;
             RemoveRateLimiterDestination(tmpDestination);
             currentSource->destinationsList = NULL;
      }
      else {
	     currentDestination = currentDestination->nextNode;
             currentSource->numberOfConnections--;
             currentSource->numberOfDestinations--;
             RemoveRateLimiterDestination(tmpDestination);
          }
    }

        currentSource->destinationsList = NULL;

}



RateLimiterSourceInfo *NewRateLimiterSource(Packet * p, FloodType floodType, int mask)
{
    RateLimiterSourceInfo *newSource = (RateLimiterSourceInfo *) malloc(sizeof(RateLimiterSourceInfo));
    if (newSource == NULL)
      {
          printf("unable to allocate memory for the newSource Rate structure \n");
       }

    newSource->prevNode = NULL;
    newSource->nextNode = NULL;
    newSource->saddr = p->iph->ip_src;
    newSource->numberOfConnections = 1;
    newSource->firstPacketTime.tv_sec = p->pkth->ts.tv_sec;
    newSource->firstPacketTime.tv_usec = p->pkth->ts.tv_usec;
    newSource->floodType = floodType;
    newSource->packetThreshold = rateLimitList[0];
    newSource->floodDetected = 0;
    newSource->subnet = mask;

  if(floodType == sUDP)
    {
        newSource->totalNumberOfTCPConnections = 0;
        newSource->totalNumberOfUDPConnections = 1;
        newSource->totalNumberOfICMPConnections = 0;
#ifdef DEBUG
        printf(MODNAME ": NewRateLimiterSource(): %s->totalnumberOfUDPConnections = 1, TCP = 0\n", inet_ntoa(newSource->saddr));
#endif

    }
    else if(floodType == sICMP)
    {
        newSource->totalNumberOfTCPConnections = 0;
        newSource->totalNumberOfUDPConnections = 0;
        newSource->totalNumberOfICMPConnections = 1;

#ifdef DEBUG
        printf(MODNAME ": NewRateLimiterSource(): %s->totalnumberOfICMPConnections = 1, TCP = 0\n", inet_ntoa(newSource->saddr));
#endif

    }
    else
    {
        newSource->totalNumberOfTCPConnections = 1;
        newSource->totalNumberOfUDPConnections = 0;
        newSource->totalNumberOfICMPConnections = 0;

#ifdef DEBUG
        printf(MODNAME ": NewRateLimiterSource(): %s->totalNumberOfTCPConnections = 1, UDP = 0\n", inet_ntoa(newSource->saddr));
#endif

    }

    newSource->destinationsList = NewRateLimiterDestination(p, floodType);
    newSource->numberOfDestinations = 1;

    return (newSource);
}

RateLimiterSourceInfo *AddRateLimiterSource(RateLimiterSourceInfo * currentSource, Packet * p, FloodType floodType, int mask)
{
    if(currentSource->nextNode)
        FatalError(MODNAME ":  AddRateLimiterSource():  Not at end of source list!");

#ifdef DEBUG
    printf(" AddRateLimiterSource Invoked");
#endif

    currentSource->nextNode = NewRateLimiterSource(p, floodType, mask);
    currentSource->nextNode->prevNode = currentSource;

    return(currentSource->nextNode);
}

void RemoveRateLimiterSource(RateLimiterSourceInfo * delSource)
{
    /*
     * If there is a prev and/or next node, make them point to the proper
     * places.  Otherwise, just delete this node.
     */
    if(delSource->prevNode || delSource->nextNode)
    {
        if(delSource->prevNode)
        {
            delSource->prevNode->nextNode = delSource->nextNode;
        }
        else if(delSource->nextNode)
        {
            delSource->nextNode->prevNode = NULL;
        }
        if(delSource->nextNode)
        {
            delSource->nextNode->prevNode = delSource->prevNode;
        }
        else if(delSource->prevNode)
        {
            delSource->prevNode->nextNode = NULL;
        }
    }
    free(delSource);
}

int updateRateLimiterConnection(RateList *rateList, Packet *p, FloodType floodType)
{
  RateLimiterSourceInfo *currentSource = rateList->listHead;
  RateLimiterDestinationInfo * currentDestination;

  struct in_addr saddr;
  struct in_addr daddr;
  u_short sport;
  u_short dport;
  int matchFound = 0;

  /* If list is empty, create the list and add this entry. */
  if(!rateList->listHead)
  {
   return 0;
  }

  if (floodType != sICMP )
      ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 1);
  else
      ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 0);

 while(!matchFound)
    {
      if( inSubnet((u_long) ntohl(currentSource->saddr.s_addr), (u_long) ntohl(saddr.s_addr), currentSource->subnet) == 1)
        {

          printf("match found with subnet %d \n", currentSource->subnet);
            currentDestination = currentSource->destinationsList;

            if(currentSource->destinationsList == NULL)
            {
                currentSource->destinationsList = NewRateLimiterDestination(p, floodType);
                currentSource->numberOfConnections ++;
#ifdef DEBUG
                printf("\n current source destination null  TCP(%d) UDP(%d) ICMP(%d) \n", currentSource->totalNumberOfTCPConnections, currentSource->totalNumberOfUDPConnections, currentSource->totalNumberOfICMPConnections);
#endif
                if(floodType == sUDP)
                {
                    currentSource->totalNumberOfUDPConnections++;

#ifdef DEBUG
                    printf(MODNAME ": UpdateRateLimiterConnection(): %s->totalNumberOfUDPConnections++ (%d)\n",
                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfUDPConnections);
#endif

                }
                else if(floodType == sICMP)
                {
                    currentSource->totalNumberOfICMPConnections++;

#ifdef DEBUG
                    printf(MODNAME ": NewRateLimiterConnection(): %s->totalNumberOfICMPConnections++ (%d)\n",
                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfICMPConnections);
#endif

                }
                else
                {
                    currentSource->totalNumberOfTCPConnections++;

#ifdef DEBUG
                    printf(MODNAME ": NewRateLimiterConnection(): %s->totalNumberOfTCPConnections++ (%d)\n",
                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfTCPConnections);
#endif

                }

                currentSource->numberOfDestinations++;
                matchFound = 1;
            }

            currentDestination = currentSource->destinationsList;

           while(!matchFound)
            {
                if(currentDestination->daddr.s_addr == daddr.s_addr)
                {
                        /*
                         * There should be error checking for
                         * currentDestination == NULL, but that should never
                         * happen.
                         */

                        if(currentDestination == NULL)
                            FatalError(MODNAME ": currentDestination is NULL!!!??\n");

                        if(currentDestination->dport == dport)
                        {
#ifdef DEBUG
                          printf("\n current source Existing TCP(%d) UDP(%d) ICMP(%d) \n", currentSource->totalNumberOfTCPConnections, currentSource->totalNumberOfUDPConnections, currentSource->totalNumberOfICMPConnections);
#endif

                          currentDestination->sport = sport;

                                if(floodType == sUDP)
                                {
                                    currentSource->totalNumberOfUDPConnections++;
                                    currentDestination->numberOfUDPConnections++;


                        #ifdef DEBUG
                                   printf("number of udp connections %d \n", currentConnection->numberOfUDPConnections);
                            printf(MODNAME ": Match NewFlood(): %s->totalNumberOfUDPConnections++ (%d)\n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfUDPConnections);
                        #endif


                                }
                                else if(floodType == sICMP)
                                {
                                    currentSource->totalNumberOfICMPConnections++;
                                    currentDestination->numberOfICMPConnections++;

                                    #ifdef DEBUG
                            printf(MODNAME ": Match NewRateLimiterConnection(): %s->totalNumberOfICMPConnections++ (%d) dport = %d \n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfICMPConnections, dport);
                                    #endif

                                }
                                else
                                {
                                    currentSource->totalNumberOfTCPConnections++;
                                    currentDestination->numberOfTCPConnections++;

                                #ifdef DEBUG
                            printf(MODNAME ": Match NewRateLimiterConnection(): %s->totalNumberOfTCPConnections++ (%d)\n",
                                   inet_ntoa(currentSource->saddr), currentSource->totalNumberOfTCPConnections);
                                #endif

                                }

                            matchFound = 1;
                        }
                        else
                        {
                            /*
                             * If not at end of list, keep going, otherwise
                             * create a node.
                             */
                            if(!currentDestination->nextNode)
                            {
#ifdef DEBUG
                               printf("\n current source destination new  TCP(%d) UDP(%d) ICMP(%d) \n", currentSource->totalNumberOfTCPConnections, currentSource->totalNumberOfUDPConnections, currentSource->totalNumberOfICMPConnections);
#endif
                                currentDestination = AddRateLimiterDestination(currentDestination, p, floodType);
                                currentSource->numberOfConnections++;
                                currentSource->numberOfDestinations++;

                                if(floodType == sUDP)
                                {
                                    currentSource->totalNumberOfUDPConnections++;

#ifdef DEBUG
                                    printf(MODNAME ": NewRateLimiterConnection(): %s->numberOfUDPConnections++ (%d)\n",
                                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfUDPConnections);
#endif

                                }
                                else if(floodType == sICMP)
                                {
                                    currentSource->totalNumberOfICMPConnections++;

#ifdef DEBUG
                                    printf(MODNAME ": NewRateLimiterConnection(): %s->totalNumberOfICMPConnections++ (%d)\n",
                                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfICMPConnections);
#endif
                                }
                                else
                                {
                                    currentSource->totalNumberOfTCPConnections++;

#ifdef DEBUG
                                    printf(MODNAME ": NewRateLimiterConnection(): %s->totalNumberOfTCPConnections++ (%d)\n",
                                           inet_ntoa(currentSource->saddr), currentSource->totalNumberOfTCPConnections);
#endif

                                }
                                matchFound = 1;
                            }
                            else
                                currentDestination = currentDestination->nextNode;
                        }
                    }
            }
        }
        else if(!currentSource->nextNode)
	{
		matchFound = 1;
	}
		
        else
          currentSource = currentSource->nextNode;
    }

    rateList->lastSource = currentSource;

    return MAX(currentSource->totalNumberOfTCPConnections, currentSource->totalNumberOfUDPConnections, currentSource->totalNumberOfICMPConnections);
}


void AddRateLimiterConnection(RateList *rateList, Packet *p, FloodType floodType, int mask)
{
  int matchFound = 0;
  RateLimiterSourceInfo *currentSource = rateList->listHead;


  /* If list is empty, create the list and add this entry. */
  if(!rateList->listHead)
  {
    rateList->listHead = NewRateLimiterSource(p, floodType, mask);
    rateList->numberOfSources = 1;
    rateList->lastSource = rateList->listHead;
    return;
  }


  while(!matchFound)
  {
   if(!currentSource->nextNode)
     {
       currentSource = AddRateLimiterSource(currentSource, p, floodType, mask);
       currentSource->numberOfConnections = 1;
       rateList->numberOfSources++;
       matchFound = 1;
     }
      else
         currentSource = currentSource->nextNode;
  }

 return ;
}

int checkInRateConnections (RateList *rateList, Packet *p, FloodType floodType)
{
  RateLimiterSourceInfo *currentSource = rateList->listHead;

  struct in_addr saddr;
  struct in_addr daddr;
  u_short sport;
  u_short dport;
  int matchFound = 0;

  if(!rateList->listHead)
  {
   return matchFound;
  }

  if (floodType != sICMP )
      ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 1);
  else
      ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 0);

  while(currentSource)
  {
   if( inSubnet((u_long) ntohl(currentSource->saddr.s_addr), (u_long) ntohl(saddr.s_addr), currentSource->subnet) == 1)
     {
       matchFound = 1;
       break;
     }

    currentSource = currentSource->nextNode;
  }

   return matchFound;


}

#if 0
int rateCheckSubnetIndFlood(RateList * rateList, Packet * p, FloodType floodType)
{
    struct in_addr saddr;
    struct in_addr daddr;
    u_short sport;
    u_short dport;
    RateLimiterSourceInfo *currentSource = rateList->listHead;
    RateLimiterDestinationInfo *currentDestination;
    int matchFound = 0;
    
    if(!rateList->listHead)
    {
      return 0;
    }
  

   if (floodType != sICMP )
        ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 1);
    else
        ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 0);

   while(!matchFound)
    {
     if( inSubnet((u_long) ntohl(currentSource->saddr.s_addr), (u_long) ntohl(saddr.s_addr), currentSource->subnet) == 1) 
      {
        currentDestination = currentSource->destinationsList;

        if(currentSource->destinationsList == NULL)
        {
          matchFound = 1;
        }

        while(!matchFound)
        {
         if (currentDestination == NULL) 
             matchFound = 1;

          if (!matchFound)
          {
           if(currentDestination->daddr.s_addr == daddr.s_addr)
             {
              matchFound = 1;
             }
             else
               currentDestination = currentDestination->nextNode;
          }
        }

      }else if (currentSource == NULL)
         {
                matchFound = 1;
         }
          else
            currentSource = currentSource->nextNode;
  }

 rateList->lastSource = currentSource;

 if (currentSource != NULL && currentDestination != NULL )
      return MAX(currentDestination->numberOfTCPConnections, currentDestination->numberOfUDPConnections, currentDestination->numberOfICMPConnections);
 
 return 0;

}

int rateCheckSubnetFlood(RateList * rateList, Packet * p, FloodType floodType)
{
    struct in_addr saddr;
    struct in_addr daddr;
    u_short sport;
    u_short dport;
    RateLimiterSourceInfo *currentSource = rateList->listHead;
    int matchFound = 0;
    
    if(!rateList->listHead)
    {
      return 0;
    }
  

   if (floodType != sICMP )
        ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 1);
    else
        ExtractFloodHeaderInfo(p, &saddr, &daddr, &sport, &dport, 0);

   while(!matchFound)
    {
     if( inSubnet((u_long) ntohl(currentSource->saddr.s_addr), (u_long) ntohl(saddr.s_addr), currentSource->subnet) == 1) 
      {
          matchFound = 1;

      }else if (currentSource == NULL)
         {
                matchFound = 1;
         }
          else
            currentSource = currentSource->nextNode;
    }

 rateList->lastSource = currentSource;

 if (currentSource != NULL)
     return MAX(currentSource->totalNumberOfTCPConnections, currentSource->totalNumberOfUDPConnections, currentSource->totalNumberOfICMPConnections); 

 return 0;

}

#endif

void ExpireRateConnections(RateList * rateList, struct spp_timeval watchPeriod, struct spp_timeval currentTime)
{
  RateLimiterSourceInfo *currentSource = rateList->listHead;
  RateLimiterDestinationInfo *currentDestination;
  RateLimiterDestinationInfo *tmpDestination;

  int matchFound = 0;

   /* Empty list. Get out of here. */
    if(!rateList->listHead)
        return;

    while(currentSource != NULL)
    {
       /*
       * If this source host is flooding us, we don't want to lose any
       * connections so go back to top.
       */
        if(currentSource->floodDetected)
        {
            currentSource = currentSource->nextNode;
            continue;
        }

      if(currentSource->firstPacketTime.tv_sec + watchPeriod.tv_sec < currentTime.tv_sec)
        {

        currentDestination = currentSource->destinationsList;

    while(!matchFound)
    {
      tmpDestination = currentDestination;
      

      if(currentDestination == NULL)
      {
        currentSource->numberOfConnections = 1;
        matchFound = 1;
      }else if(currentDestination->nextNode == NULL) {
        matchFound = 1;
        currentSource->numberOfConnections--;
        currentSource->numberOfDestinations--;
        RemoveRateLimiterDestination(tmpDestination);
      }
      else {
	     currentDestination = currentDestination->nextNode;
             currentSource->numberOfConnections--;
             currentSource->numberOfDestinations--;
             RemoveRateLimiterDestination(tmpDestination);
          }
    }
        currentSource->destinationsList = NULL;

         rateList->numberOfSources--;
#if 0
         RemoveRateLimiterSource(currentSource);
#endif
        }
       if (currentSource != NULL)
           currentSource = currentSource->nextNode;
    }


   if(rateList->numberOfSources == 0)
    {
        printf("rateList->numberOfSources == %d \n",rateList->numberOfSources);
        rateList->listHead = NULL;
    }

}


void CreateRateLimiterList(u_char *args)
{
   char **toks;
    int num_toks;
    int i;

    rateList  = CreateRateList();

   rateMod = oN;

    /* tokenize the argument list */
    toks = mSplit(args, " ", 31, &num_toks, '\\');

    reportTime.tv_sec = atoi(toks[0]);
    reportTime.tv_usec = 0;

    rateLimitList = (int *) calloc(num_toks, sizeof(int));

    for(i = 1; i < num_toks; i++)
    {
        rateLimitList[i-1] = atoi(toks[i]);
    }

    rateLevels = num_toks - 1;

#ifdef DEBUG
    for(i = 0; i < (num_toks - 1); i++)
    {
        printf(" token[%d] = %d \n", i, rateLimitList[i]);
    }
#endif

    for(i = 0; i < num_toks; i++)
    {
        free(toks[i]);
    }

}

void SetupFloodRateLimiter(void)
{
    RegisterPreprocessor("flood-ratelimiter", FloodRateLimiterInit);
}

void FloodRateLimiterInit(u_char * args)
{
    CreateRateLimiterList(args);
}

