#define INITGUID 1

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <signal.h>
#include <time.h>
#include "pntypes.h"
#include "pnresult.h"
#include "pncom.h"
#include "rmappv.h"
#include "rmaplugn.h"
#include "ppvtime.h"
#include "ppvmd5.h"
#include "ppvini.h"
#include "os.h"
extern "C" {
#include "cgi-lib.h"
#include "html-lib.h"
};
#include "ppvdemo.h"

// Local Helper Functions
int MakeRegisterUrl( char* szRegisterUrl, char* szIniCfg, char* szUserID );
int GetIniCfg( char* szIniCfg);

IRMAPPVDatabase*  g_pPPV  = NULL;

// If your database is password protected, fill in the following Username 
// and Password strings.  These strings will only be used to open the 
// database and will NOT be accessible to anyone after the cgi is built. 

const char SZ_DB_USERID[50]   = "Replace this text with database username";
const char SZ_DB_PASSWORD[50] = "Replace this text with database password";

int main()
{
	llist 				form_input;					// linked list to hold form info
   
	int					nResult; 
	
	HINSTANCE			DLL;
	FPCREATEINSTANCE	fpCreateInstance;
 	IUnknown*			pUnknown = NULL;

	char*				szCommand;
	char				szPluginName[BUF_LENGTH];
	char				szDatabase[BUF_LENGTH];
	char				szIniCfg[BUF_LENGTH];
	char				szBuf[BUF_LENGTH];			// buffer variable
  
	// retrieve file AND form data
	read_cgi_input(&form_input);
	
	// retrieve required INI configuration file
	if( !GetIniCfg( szIniCfg ) )
	{
		list_clear(&form_input);
        	return 0;
	}

	// retrieve plugin name
	GetPrivateProfileString( INI_SYSTEM, INI_PLUGIN, EMPTY_STRING, szBuf, sizeof( szBuf), szIniCfg );

	// copy buffer contents
	strcpy( szPluginName, szBuf );
	// empty string buffer
	memset( szBuf, 0, sizeof( szBuf ) );	
	
	// retrieve database name
	GetPrivateProfileString( INI_SYSTEM, INI_DATABASE, EMPTY_STRING, szBuf,sizeof( szBuf), szIniCfg );

	// copy buffer contents
	strcpy( szDatabase, szBuf );
	
	// load plugin library
	DLL = LoadLibrary( szPluginName );

	// confirm if this is an RMA Plugin
	fpCreateInstance =
		( FPCREATEINSTANCE )GetProcAddress( DLL, "RMACreateInstance" );

	// error checking for failure of plugin loading
	if ( !fpCreateInstance )
	{
		print_begin();
		printf("<h2>Plugin %s failed to load.</h2>", szPluginName );
		print_end();
		list_clear(&form_input);
  		return 0;
	}

	// error checking for invalid object pointer
	if (PNR_OK != fpCreateInstance(&pUnknown))
	{
		print_begin();
		printf("<h2>Plugin %s failed to create object.</h2>", szPluginName );
		print_end();
		list_clear(&form_input);
  		return 0;
	}

	// error checking for invalid interface
	if ( PNR_OK != pUnknown->QueryInterface( IID_IRMAPPVDatabase, (void**) &g_pPPV ) )
	{
		pUnknown->Release();
		print_begin();
		printf("<h2>Plugin %s failed to find PPVDatabase interface.</h2>", szPluginName );
		print_end();
		list_clear(&form_input);
  		return 0;
	}

	// Initialize Plugin   
	nResult = g_pPPV->InitPPVDB( szDatabase, SZ_DB_USERID, SZ_DB_PASSWORD );
	
	// handling failure to initialize database
	if ( PNR_PPV_AUTHORIZATION_FAILED == nResult )
	{
		print_begin();
        	printf("<h2>Error opening database.  Invalid userid/password combination.</h2>");
        	print_end();
        	list_clear(&form_input);
        	return 0;
	}
	
	// handling failure to open database
	else if ( PNR_OK != nResult )
	{
		print_begin();
		printf("<h2>Error opening the database</h2>");
		print_end();
		list_clear(&form_input);
		return 0;
	}
   
	// retreive command
	szCommand = cgi_val( form_input, FORMNAME_COMMAND );
	
	if ( 0 != szCommand )
	{
		// insert a user
		
		if ( 0 == strcmp( szCommand, FORMVALUE_INSERTUSER )  )
			HandleInsertUser( &form_input, szIniCfg );
		
		// grant the permission
		
		else if ( 0 == strcmp( szCommand, FORMVALUE_GRANTACCESS ) )
			HandleGrantAccess( &form_input, szIniCfg );
		
		else 
		
		// handling invalid form value; command
		{
			print_begin();
			printf("<h2>Command \"%s\" unknown.</h2>", szCommand);
			print_end();
		}	
	}
	
	else
	
	// handling missing form value; command
	{
		print_begin();
		printf("<p>&nbsp;\n");
		printf("<h2>Error: Missing Command.</h2>");
		print_end();
	}

	// clear memory for form data
	list_clear(&form_input);
	g_pPPV->Release(); 
	FreeLibrary(DLL);
	return 0;
}



//	HandleInsertUser
//	Description:  inserts a user & password, if relevant


void HandleInsertUser

(
	llist*		form_input,
	char*		szIniCfg
) 
{
	char*		szUserid;
	char*		szPassword;
	char		szRegisterUrl[BUF_LENGTH];
	char		szPermissionsUrl[BUF_LENGTH];
	char		szBuf[BUF_LENGTH];
	char		szAuthMode[BUF_LENGTH];
	char		szRealm[BUF_LENGTH];
   
	int			nResult;
	PPVRegLog	log_item;
	char		szNull[3] = " ";
	char		temp[2048];
	char		authKey[2048];

	// error checking for valid user ID input 
	szUserid = cgi_val(*form_input, FORMNAME_USERID );
	if ( 0 == szUserid )
	{
		print_begin();
		printf("<h2>A userid must be specified.</h2>");
		print_end();
		return;
	}

	// error checking for valid password input 
	szPassword = cgi_val(*form_input, FORMNAME_PASSWORD );
	if ( 0 == szPassword )
	{
		print_begin();
		printf("<h2>A password must be specified.</h2>");
		print_end();
		return;
	}

	// retrieve server realm
	GetPrivateProfileString( INI_SYSTEM, INI_REALM, EMPTY_STRING, szBuf, sizeof( szBuf ), szIniCfg );
	strcpy( szRealm, szBuf );

	// retrieve server authentication mode
	GetPrivateProfileString( INI_SYSTEM, INI_AUTHMODE, EMPTY_STRING, szBuf,sizeof(szBuf), szIniCfg);
	strcpy( szAuthMode, szBuf );

	// retrieve server permissions URL
	GetPrivateProfileString( INI_URLS, INI_PERMISSIONS, EMPTY_STRING, szBuf, sizeof(szBuf), szIniCfg);
	strcpy( szPermissionsUrl, szBuf );
	
	// construct server player registration URL
	if ( 0 == strcmp(szAuthMode, "0") )
		MakeRegisterUrl( szRegisterUrl , szIniCfg, szUserid );

	// remove user to prevent conflicts
	g_pPPV->RemoveUser(szUserid);

	// construct password
	sprintf(temp, "%s:%s:%s", szUserid, szRealm, szPassword);
	MD5Data(authKey, (const unsigned char *)temp, strlen(temp));

	// insert user with password set in 'authKey'
	nResult = g_pPPV->InsertUser(szUserid, authKey);
   
	if ( PNR_OK != nResult )
	{
		// an error occurred
		print_begin();
		printf("<h2>Error: an internal error occurred.</h2>");
		print_end();
		return;
	}
   
	if ( 0 == strcmp(szAuthMode, "0") )
	{
		// submit multipart header
		printf("Content-type: multipart/x-mixed-replace;boundary=End\n\n");
		printf("--End\n");

		// submit audio header
		printf("Content-type: audio/x-pn-realaudio\n\n");
		printf("%s\n", szRegisterUrl);
		printf("--End\n");

		// submit html header
	    printf("Content-type: text/html\n\n");
        printf("<html> <META HTTP-EQUIV=REFRESH "
					"CONTENT='0;URL=%s'> ", szPermissionsUrl );
		printf("--End\n");

		// terminate multipart header
		printf("--End--\n");

		return;
   }
   else
   {
		// log registration
		// fill out the log item and send it to the database
		log_item.nStatus = (PPVRegStatus)0;
		log_item.pUserid = szUserid;
		log_item.pGUID = szNull;
		log_item.pIPAddress = szNull;
		time(&(log_item.tRequestTime));
		log_item.pURLRedirect = szNull;

		if ( PNR_OK != g_pPPV->LogReg(&log_item) ) 
        {
			// an error occurred
			print_begin();
			printf("<h2>Error: an internal error occurred while logging to the database.</h2>");
			print_end();
			return;
		}

        printf("Content-type: text/html\n\n");
        printf("<html> <META HTTP-EQUIV=REFRESH "
					"CONTENT='0;URL=%s'> ", szPermissionsUrl);
		print_end();
		return;
	}
}


// HandleGrantAccess
// Description:  grants user access 

void HandleGrantAccess
(
	llist*		form_input,
	char*		szIniCfg
) 
{
	char*		szUserid;						// re userid
	char*		szPermissionType;				// re permission
	char*		szUrl;							// re url
	char*		szUrlType;						// re urltype
	char*		szPermissionLength;				// re debittime/expirestime
	char		szPermissionLengthFormatted[2048];
	char		szRedirect[128];
	char		szBuf[64];
	
	PPVPermission	permission;
	int				nResult;

	// retrieve current directory
	GetCurrentDirectory( sizeof( szIniCfg ), szIniCfg);

	GetPrivateProfileString( INI_URLS, INI_REDIRECT, EMPTY_STRING,szBuf, sizeof(szBuf), szIniCfg );
	strcpy( szRedirect, szBuf );
	
	szUserid = cgi_val(*form_input, FORMNAME_USERID);
	if ( 0 == szUserid )
	{
		print_begin();
		printf("<h2>You must specify a userid.</h2>");
		print_end();
		return;
	}

    // checks for valid url for secure directory
	szUrl = cgi_val(*form_input, FORMNAME_URL );
    if ( 0 == szUrl )
    {
 		print_begin();
		printf("<h2>You must specify an url to grant access.</h2>");
		print_end();
		return;
    }

    // checks for urltype
    szUrlType = cgi_val(*form_input, FORMNAME_URLTYPE);
    if ( 0 == szUrlType )
    {
		print_begin();
        printf("<h2>You must specify an urltype of 0 (file) or 1 (directory).</h2>");
        print_end();
        return;
    }
    
	// 	checks for valid urltype
	if ( 0 == strcmp(szUrlType, "0"))
        permission.nURLType = (PPVURLType)0;
	else if ( 0 == strcmp(szUrlType, "1"))
		permission.nURLType = (PPVURLType)1;
    else
    {
        print_begin();
        printf("<h2>You must specify an urltype of 0 (file) or 1 (directory).</h2>");
        print_end();
        return;
    }
	
    // checks for valid type of permission granted
    szPermissionType = cgi_val(*form_input, FORMNAME_PERMISSIONTYPE );
    if ( 0 == szPermissionType )
    {
 		print_begin();
		printf("<h2>You must specify a permission type.</h2>");
		print_end();
		return;
    }
  
    // process permission instructions
	if ( 0 == strcmp(szPermissionType, "0") )	//general access
    {
		strcpy(permission.pURL,szUrl);
	
		permission.nPermissionType = (PPVPermissionType)0;
		permission.tExpires = 0;
		permission.ulDebitTime = 0;
	
		// TIP:  Uncomment the following to ensure that users have access to
		//       one content piece only
		//g_pPPV->RevokeAllPermissions(szUserid);

		nResult = g_pPPV->GrantPermission(szUserid, &permission);

		if ( PNR_OK != nResult )
		{
			print_begin();
			printf("<h2>Unable to grant permission. An internal error occurred.</h2>");
			print_end();
			return;
		} 
	}
	else if ( 0 == strcmp(szPermissionType, "1") ) // expiration based access
    {
		szPermissionLength = cgi_val(*form_input, FORMNAME_PERMISSIONLENGTH );
		if ( 0 == szPermissionLength )
		{
			print_begin();
			printf(	"<h2>You must specify an expiration time to "
					"grant an expiration permission.</h2>");
			print_end();
			return;
		}
	
		// to save the user from typing in hours, minutes, seconds.
		strcpy(szPermissionLengthFormatted, szPermissionLength);
		strcat(szPermissionLengthFormatted, ":00:00:00");
		strcpy(permission.pURL, szUrl);
		permission.nPermissionType = (PPVPermissionType)1;
		ppvStringToTime(szPermissionLengthFormatted, &permission.tExpires);
		permission.ulDebitTime = 0;
		
		// TIP:  Uncomment the following to ensure that users have access to
		//       one content piece only
		//g_pPPV->RevokeAllPermissions(szUserid);

		nResult = g_pPPV->GrantPermission(szUserid, &permission);
		if ( PNR_OK != nResult )
		{
			print_begin();
			printf(	"<h2>Unable to grant permission. An internal error "
					"occurred.</h2>");
			print_end();
			return;
		} 
    }

	else if ( 0 == strcmp(szPermissionType, "2") ) // debit based access, in seconds
	{
 		szPermissionLength  = cgi_val(*form_input, FORMNAME_PERMISSIONLENGTH );
		if ( 0 == szPermissionLength )
		{
			print_begin();
			printf(	"<h2>You must specify a debit time to "
					"grant a debit permission.</h2>");
			print_end();
			return;
		}

		if ( 0 >= atol(szPermissionLength) )
		{
			print_begin();
			printf("<h2>You must specify a time greater than 0.</h2>");
			print_end();
			return;
		}

		strcpy(permission.pURL, szUrl);
		permission.nPermissionType = (PPVPermissionType)2;
		permission.tExpires = 0;
		permission.ulDebitTime = atol(szPermissionLength);
	
		// TIP:  Uncomment the following to ensure that users have access to
		//       one content piece only
		//g_pPPV->RevokeAllPermissions(szUserid);

		nResult = g_pPPV->GrantPermission(szUserid, &permission);
		if ( PNR_OK != nResult )
		{	
			print_begin();
			printf(	"<h2>Unable to grant permission. An internal error "
					"occurred.</h2>");
			print_end();
			return;
		} 
    }
	else if ( 0 == strcmp(szPermissionType, "3") )	//credit access
    {
		strcpy(permission.pURL,szUrl);
	
		permission.nPermissionType = (PPVPermissionType)3;
		permission.tExpires = 0;
		permission.ulDebitTime = 0;
	
		// TIP:  Uncomment the following to ensure that users have access to
		//       one content piece only
		//g_pPPV->RevokeAllPermissions(szUserid);

		nResult = g_pPPV->GrantPermission(szUserid, &permission);

		if ( PNR_OK != nResult )
		{
			print_begin();
			printf("<h2>Unable to grant permission. An internal error occurred.</h2>");
			print_end();
			return;
		} 
	}
    else
    {
 		print_begin();
		printf("<h2>Not a valid permission type.</h2>");
		print_end();
		return;
	}

    printf("Content-type: text/html\n\n");
    printf(	"<html><META HTTP-EQUIV=REFRESH "
			"CONTENT='0;URL=%s'>", szRedirect);
	return;
}


// print_header
// Description:  print the header, title, body tags, etc

void print_begin
(
)
{
	html_header();	   // print the header
   
	printf("<html>\n");	

	// print the body and title tags
	printf("<title>Pay-Per-View Demo Results</title>\n");
	printf("<body bgcolor=\"6666FF\" text=\"black\" ");
	printf(" link=\"white\" vlink=\"white\">\n");
}

// print_end
// Description: print the footer and end body tags

void print_end
(
)

{
   printf("<hr>\n");
   printf("</body></html>");
}

int GetIniCfg( char* szIniCfg)
{
	char		szBuf[BUF_LENGTH];

	// retrieve current directory
	GetCurrentDirectory( sizeof( szBuf ), szBuf);

	// copy buffer contents
	strcpy( szIniCfg, szBuf );

	// error checking for invalid INI configuration file
	if ( strcmp( szIniCfg, EMPTY_STRING) == 0 )
	{
		print_begin();
        printf("<h2>Configuration file \"INI_CONFIG\" not found.</h2>");
        print_end();
		return 0;
	}

	// append slash character respective to platform
#ifdef WIN32
	strcat( szIniCfg, "\\" );
#else
	strcat( szIniCfg, "/" );
#endif
	// append INI configuration file name to construct full path name
	strcat( szIniCfg, INI_CONFIG ); 

	return 1;
}

int MakeRegisterUrl( char* szRegisterUrl, char* szIniCfg, char* szUserID )
{
	char	szBuf[BUF_LENGTH];			// buffer variable
	
	GetPrivateProfileString( INI_URLS, INI_PROTOCOL, EMPTY_STRING, szBuf,sizeof(szBuf), szIniCfg );
	strcpy( szRegisterUrl, szBuf );

	strcat( szRegisterUrl, "://" );

	GetPrivateProfileString( INI_URLS, INI_SERVER ,EMPTY_STRING, szBuf,sizeof(szBuf), szIniCfg );
	strcat( szRegisterUrl, szBuf );

	strcat( szRegisterUrl, ":" );

	GetPrivateProfileString( INI_URLS, INI_PORT ,EMPTY_STRING, szBuf,sizeof(szBuf), szIniCfg );
	strcat( szRegisterUrl, szBuf );

	strcat( szRegisterUrl, "/" );

	GetPrivateProfileString( INI_URLS, INI_PREFIX ,EMPTY_STRING, szBuf,sizeof(szBuf), szIniCfg );
	strcat( szRegisterUrl, szBuf );

	strcat( szRegisterUrl, szUserID );

	strcat( szRegisterUrl, "!" );

	GetPrivateProfileString( INI_URLS, INI_FILE, EMPTY_STRING, szBuf,sizeof(szBuf), szIniCfg );
	strcat( szRegisterUrl, szBuf );

	return 1;
}

