
function AccessRule ( addressCtrl, netmaskCtrl, ruleCtrl, propList )
{
	this.m_addressCtrl = addressCtrl ;
	this.m_netmaskCtrl = netmaskCtrl ;
	this.m_ruleCtrl = ruleCtrl ;
	this.m_propList = propList ;
	this.m_bAllowHostnames = true ;
	this.m_bAllowZero = false ;

	return this ;
}

AccessRule.prototype.AllowHostnames = function ( bAllowHostnames )
{
	if ( arguments.length )
	{
		this.m_bAllowHostnames = ( bAllowHostnames ? true : false );
	}
	return this.m_bAllowHostnames ;
}

// allow the address 0.0.0.0
AccessRule.prototype.AllowZero = function ( bAllowZero )
{
	if ( arguments.length )
	{
		this.m_bAllowZero = ( bAllowZero ? true : false );
	}
	return this.m_bAllowZero ;
}

AccessRule.prototype.split = function ()
{
	// blank out the fields first
	setCtrlValue( this.m_addressCtrl, "" );
	setCtrlValue( this.m_netmaskCtrl, "" );

	var rule = getCtrlValue( this.m_ruleCtrl );
	if ( ! rule ) return ;

	var tempA = rule.split( /[\/:]/ );
	if ( tempA && tempA[ 0 ] )
	{
		// if the address is missing, split() will put the netmask in the
		// first array slot
		if ( rule.bSearch( /^ *[\/:]/ ) )
		{
			setCtrlValue( this.m_netmaskCtrl, tempA[ 0 ] );
		}
		else
		{
			setCtrlValue( this.m_addressCtrl, tempA[ 0 ] );

			if ( tempA[ 1 ] )
				setCtrlValue( this.m_netmaskCtrl, tempA[ 1 ] );
		}
	}
}

AccessRule.prototype.validate = function ()
{
	// validate address as a proper ip address
	// stores in rule control if valid

	errMsgIP = "IP Addresses must be of the form [1-255].[0-255].[0-255].[0-255]" ;

	var bValid = false ;
	var addr = getCtrlValue( this.m_addressCtrl );
	var rule = addr ;

	// if the address is an IP address, makes sure its valid.
	// then, determine wether a netmask is needed
	if ( ! isIPAddrFormat( addr ) )
	{
		// do we allow hostnames?
		if ( ! this.m_bAllowHostnames )
		{
			return rejectInput( this.m_addressCtrl, errMsgIP );
		}

		// see if the hostname is valid
		if ( ! this.m_propList.hostname_validate( addr ) )
			return rejectInput( this.m_addressCtrl, this.m_propList.errMsg );

		// else hostname is good - no netmask needed
	}
	else if ( !isIPAddr( addr, this.m_bAllowZero ) )
	{
		// invalid IP Address
		return rejectInput( this.m_addressCtrl, errMsgIP );
	}
	else	// valid IP Address - see if the netmask is valid (and whether its needed or not)
	{
		// determine if a netmask is required: the address has a zero in it
		var needNetmask = false ;
		var octets = addr.split( '.' );
		for ( var i = 0; i < octets.length; i++ )
		{ 
			if ( parseInt( octets[ i ] ) == 0 )
			{
				needNetmask = true ;
				break ;
			}	
		}

		if ( needNetmask )
		{
			var netmask = getCtrlValue( this.m_netmaskCtrl );
			if ( ! netmask )
			{
				return rejectInput( this.m_netmaskCtrl, "A netmask is required for this IP address." );
			}

			if ( ! this.m_propList.netmask_validate( netmask ) )
				return rejectInput( this.m_netmaskCtrl, this.m_propList.errMsg );

			var seperator = ":" ;
			if ( -1 == netmask.indexOf( "." ) )
				seperator = "/" ;

			rule += seperator + netmask ;
		}
	}

	setCtrlValue( this.m_ruleCtrl, rule );

	return true ;
}

AccessRule.prototype.join = function ()
{
	var seperator = ":" ;

	var netmask = getCtrlValue( this.m_netmaskCtrl );
	if ( netmask == "" )
		seperator = "" ;
	else if ( parseInt( netmask ) == netmask )
		seperator = "/" ;

	setCtrlValue( this.m_ruleCtrl, 
		getCtrlValue( this.m_addressCtrl ) +
		seperator + netmask );
}

/* RuleEditSelect is a subclass of the EditSelect class, done so we can override the
	getNextUntitledIncrement funtion()
*/
function RuleEditSelect ( selectCtrl, editCtrl, validateFunc, uniqOnly /* optional == true */ )
{
	this.base = EditSelect ;
	this.base( selectCtrl, editCtrl, validateFunc, uniqOnly );
}
RuleEditSelect.prototype = new EditSelect ;

/* use the next highest hundred, instead of incrementing by one */
RuleEditSelect.prototype.getNextUntitledIncrement = function ( curMax )
{
	return ( curMax + 100 - ( curMax % 100) );
}

