
function EditSelect ( selectCtrl, editCtrl, validateFunc, uniqOnly /* optional == true */, editFunc /* optional == null */ )
{
	if ( ! ( selectCtrl && editCtrl ) )
		return null ;

	if ( (selectCtrl.options == null) || (editCtrl.value == null) )
		return null ;

	this.selectCtrl = selectCtrl ;
	this.editCtrl = editCtrl ;
	this.validateFunc = validateFunc ;
	this.uniqOnly = (uniqOnly == null ? true : uniqOnly);
	this.editFunc = editFunc;
	if ( editFunc )
	{
		editCtrl.onKeyPress = this._onKeyPress ;
	}

	this.frameMain = findFrame( "Main" ); 

	return this ;
}

EditSelect.prototype.sync = function ()
{
	setCtrlValue( this.editCtrl, getCtrlValue( this.selectCtrl, true ) );
}

EditSelect.prototype.validate = function ()
{
	var s = getCtrlValue( this.editCtrl );
	if ( this.uniqOnly )
	{
		for ( var i = 0; i < this.selectCtrl.length; i++ )
		{
			if ( s == this.selectCtrl.options[ i ].value )
				return rejectInput( this.editCtrl, "New value is not unique." );
		}
	}

	if ( ! this.validateFunc )
		return true ;

	var errMsg = this.validateFunc( s );
	if ( errMsg != "" )
		return rejectInput( this.editCtrl, errMsg );
	else
		return true ;
}

EditSelect.prototype.replace = function ()
{
	var selX = this.selectCtrl.selectedIndex ;
	if ( selX == -1 )
	{
		alert( 'Use the "Add New" button first.' );
		return false ;
	}

		var s = getCtrlValue( this.editCtrl );
	if ( (selX != -1) &&
		 (s != "") && 
	     (s != getCtrlValue( this.selectCtrl )) )
	{
		if ( this.validate() )
		{
			this.selectCtrl.options[ selX ].value = s ;
			this.selectCtrl.options[ selX ].text = s ;
			return true ;
		}
	}

	return false ;
}

EditSelect.prototype.add = function ()
{
	// add to list and select
	var len	= this.selectCtrl.length ;
	var s = getCtrlValue( this.editCtrl );

	if ( (s != "") && 
	     (s != getCtrlValue( this.selectCtrl )) )
	{
		if ( this.validate() )
		{
			this.addOption( s, s );
			return true ;
		}
	}

	return false ;
}

EditSelect.prototype.addUntitled = function ( defTitle, noIncrement, titlePostFix, displayText )
{
	if ( defTitle == null ) defTitle = "Untitled" ;

	if ( ! noIncrement )
	{
		var max = 0 ;
		if ( titlePostFix == null ) 
			titlePostFix = "" ;

		// loop through the list looking for elements with the form 
		// <defTitle>[0-9]+

		var tempA = null ;
		var regExp = new RegExp( "^" + defTitle.replace( /\//g, "\\/" ) + "(\\d+)" + 
								   titlePostFix.replace( /\//g, "\\/" ) + "$", "i" );

		for ( var i = 0; i < this.selectCtrl.length; i++ )
		{
			tempA = this.selectCtrl.options[ i ].value.match( regExp );
			if ( tempA && parseInt( tempA[ 1 ] ) > max ) max = parseInt( tempA[ 1 ] );
		}

		defTitle += this.getNextUntitledIncrement( max ) + titlePostFix ; 
	}

	if ( !displayText ) 
		displayText = defTitle ;

	// add to list and select
	this.addOption( displayText, defTitle );

	setCtrlValue( this.editCtrl, displayText );
	this.editCtrl.focus();
	this.editCtrl.select();

	return true ;
}

EditSelect.prototype.getIncrementedDisplayText = function ( defTitle, increment, titlePostFix )
{
	var max = 0 ;
	if ( titlePostFix == null ) 
		titlePostFix = "" ;

	// loop through the list looking for elements with the form 
	// <defTitle>[0-9]+
	var tempA = null ;
	var regExp = new RegExp( "^" + defTitle.replace( /\//g, "\\/" ) + "(\\d+)" + 
							   titlePostFix.replace( /\//g, "\\/" ) + "$", "i" );

	for ( var i = 0; i < this.selectCtrl.length; i++ )
	{
		tempA = this.selectCtrl.options[ i ].text.match( regExp );
		if ( tempA && parseInt( tempA[ 1 ] ) > max ) max = parseInt( tempA[ 1 ] );
	}

	return defTitle + (max + increment) + titlePostFix ; 
}

EditSelect.prototype.getNextUntitledIncrement = function ( curMax )
{
	return curMax + 1 ;
}

EditSelect.prototype.remove = function ()
{
	selectDel( this.selectCtrl );
	this.sync();

	return true ;
}

EditSelect.prototype.count = function ()
{
	return ( this.selectCtrl ? this.selectCtrl.length : 0 );
}

EditSelect.prototype.addOption = function ( sText, sValue )
{
	// add to list and select
	var len	= this.selectCtrl.length ;

	if ( ! ( isIE && isIE5Up ) )
	{
		this.selectCtrl.options[ len ] = new Option( sText, sValue );
	}

	else	
	{
		if ( this.frameMain )
		{
			var oOption = this.frameMain.document.createElement( "OPTION" );
			oOption.text = sText ;
			oOption.value = sValue ;
			this.selectCtrl.options[ len ] = oOption ;
		}
	}

	this.selectCtrl.selectedIndex = len ;

}

EditSelect.prototype._onKeyPress = function ( evt )
{
	if ( this.editFunc )
	{
		var keyCode = ( isIE ) ? event.keyCode : evt.which

		switch ( keyCode )
		{
			case 13:	// ENTER
				this.editFunc();

		}
	}
	return  true ;
}


