
// IP Address checking stuff
// requires ra_client.js

var proplist		 	= new PropList();	// for IP Address validation
var validMulticastRange = new IPRange( new IPAddress('224.0.0.255') , new IPAddress('239.255.255.255') );

// raise base number to exponent function
function exp( base , exponent )
{
	if ( exponent == 0 )
		return 1;

	// don't support negative exponents
	if ( exponent < 0  )
		return 0;

	return base * exp( base , exponent - 1 );
}

// create an ip address range from a string
function stringToIPRange( str ) 
{
	var tmp = new Object;
	var addrs = new Array();
	addrs = str.split('-');

	if ( addrs.length != 2 ) {
		tmp.errMsg = "IP Ranges must be of the form [0-255].[0-255].[0-255].[0-255]-[0-255].[0-255].[0-255].[0-255].";
		return tmp;
	}

	if ( ! ( proplist.ip_validate( addrs[0] ) ) && ( proplist.ip_validate( addrs[1] ) ) ) {
		tmp.errMsg = proplist.errMsg;
		return tmp;
	}

	return new IPRange( new IPAddress( addrs[0] ) , new IPAddress( addrs[1] ) );
}

// ip address range constructor
function IPRange( addr1 , addr2 )
{
	// test to make sure we're working with IPAddress objects
	if ( addr1.type && addr2.type && addr1.type == 'IPAddress' && addr2.type == 'IPAddress')
	{
		if ( addr1.addr > addr2.addr )
		{
			this.addressLow  = addr2;
			this.addressHigh = addr1;
		} 
		else
		{
			this.addressLow	 = addr1;
			this.addressHigh = addr2;
		}
	}
	else
	{
		this.errMsg = "Bad IPRange parameters."
	}

	return this;
}

IPRange.prototype.toString = function ( )
{
	return this.addressLow.toString() + '-' + this.addressHigh.toString();
}

IPRange.prototype.addressInRange = function ( addr )
{
	return addr.isInRange( this );
}

IPRange.prototype.isOverlap = function ( range )
{
	return range.addressHigh.isInRange( this ) || range.addressLow.isInRange( this ) || this.addressHigh.isInRange( range ) || this.addressLow.isInRange( range );
}

IPRange.prototype.isSubset = function ( range )
{
	return this.addressHigh.isInRange( range ) && this.addressLow.isInRange( range );
}

IPRange.prototype.numAddrsInRange = function ( )
{
	return this.addressHigh.addr - this.addressLow.addr + 1;
}

function IPAddress( addr )
{
	if ( ! proplist.ip_validate( addr ) ) {
		this.errMsg = proplist.errMsg;
		return this;
	}

	var octets	= addr.split('.');
	if ( octets.length < 4 )
	{
		// assure we've got at least 4 octets
		octets.push( 0, 0, 0, 0 );
	}

	this.addr = eval(octets[0] * 16777216) + eval(octets[1] * 65536) + eval(octets[2] * 256) + eval(octets[3]);

	this.type = 'IPAddress';
	return this;
}

IPAddress.prototype.toString = function ( ) 
{
	var str = "";
	for ( var i = 0; i < 4; i++ ) {
		str += ( this.addr & ( 255 << ( 3 - i ) * 8 ) ) >>> ( 3 - i ) * 8;
		str += '.';
	}

	// return string without the trailing '.'
	return str.substr(0 , str.length - 1 );
}

IPAddress.prototype.isBetween = function ( compareIP1 , compareIP2 )
{
	var range = new IPRange( compareIP1 , compareIP2 );
	return this.isInRange( range );
}

IPAddress.prototype.isInRange = function ( range )
{
	return (this.addr < range.addressHigh.addr && this.addr > range.addressLow.addr) || ( this.addr == range.addressLow.addr || this.addr == range.addressHigh.addr );
}

IPAddress.prototype.isClassD = function ()
{
	return (this.addr >= 3758096384) && (this.addr < 4026531840);
}

IPAddress.prototype.isClassC = function ()
{
	return (this.addr >= 3221225472) && (this.addr < 3758096384);
}


